from src.execution import parent_parser
from collections import namedtuple


def mock_command_action():
    print("a mockery!")


# basically replaces tabcmd_controller:initialize_parsers
def initialize_test_pieces(commandname, command_object):
    manager = parent_parser.ParentParser()
    parser = manager.get_root_parser()
    mock_command = namedtuple("TestObject", "name, run_command, description, define_args")
    mock_command.name = commandname
    mock_command.run_command = mock_command_action
    mock_command.description = "mock help text"
    mock_command.define_args = command_object.define_args

    manager.include(mock_command)
    return parser


"""
 base test cases for each parser:
 has_required_arguments
 (maybe) missing required arguments
 has optional arguments
 bad mix of optional arguments
 has unknown arguments
 """



import unittest

from src.commands.extracts.create_extracts_command import CreateExtracts
from .common_setup import *

commandname = "createextracts"


class CreateExtractsParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, CreateExtracts)

    def test_create_extract_parser_missing_all_args(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_create_extract_parser_missing_project_path(self):
        mock_args = [
            commandname,
            "--project",
            "test123",
            "--workbook",
            "workbooktest",
            "--include-all",
            "test",
        ]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_create_extract_parser_mutually_exclusive_opts(self):
        mock_args = [
            commandname,
            "--project",
            "test123",
            "--workbook",
            "workbooktest",
            "--embedded-datasources",
            "desc",
            "--include-all",
            "test",
        ]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_create_extract_parser_optional_arguments(self):
        mock_args = [
            commandname,
            "--datasource",
            "testproject",
            "--project",
            "test123",
            "--parent-project-path",
            "abcdef",
            "--include-all",
        ]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.datasource == "testproject", args
        assert args.include_all is True, args

    def test_create_extract_parser_missing_workbook(self):
        mock_args = [
            commandname,
            "--workbook",  # no workbook name 'testproject',
            "--project",
            "test123",
            "--parent_project_path",
            "abcdef",
            "--embedded_datasources",
            "desc",
            "--workbook",
            "workbooktest",
            "--include_all",
        ]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_create_extract_parser_missing_datasource(self):
        mock_args = [
            commandname,
            "--datasource",  # no name 'testproject',
            "--project",
            "test123",
            "--parent-project-path",
            "abcdef",
            "--embedded-datasources",
            "desc",
            "--workbook",
            "workbooktest",
            "--include-all",
        ]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_create_extract_parser_missing_embedded_datasources(self):
        mock_args = [
            commandname,
            "--datasource",
            "testproject",
            "--project",
            "test123",
            "--parent-project-path",
            "abcdef",
            "--embedded-datasources",
        ]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)



import unittest

from src.commands.site.create_site_command import CreateSiteCommand
from .common_setup import *

commandname = "createsite"


class CreateSiteParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, CreateSiteCommand)

    def test_create_site_parser_just_a_name(self):
        mock_args = [commandname, "site-name"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.site_name == "site-name", args

    def test_create_site_parser_missing_required_name(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_create_site_parser_user_quota_integer(self):
        mock_args = [commandname, "site-name", "--user-quota"]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_create_site_parser_with_all_args(self):
        mock_args = [
            commandname,
            "site-name",
            "--user-quota",
            "12",
            "--storage-quota",
            "12",
        ]  # what else?
        args = self.parser_under_test.parse_args(mock_args)
        print(args)
        assert args.site_name == "site-name", args
        assert args.user_quota == 12, args
        assert args.storage_quota == 12, args



from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log


class PublishSamplesCommand(Server):
    """
    Command to Publish Tableau Sample workbooks to the specified project.
    Any existing samples will be overwritten.
    """

    name: str = "publishsamples"
    description: str = _("publishsamples.short_description")

    @staticmethod
    def define_args(publish_samples_parser):
        publish_samples_parser.add_argument(
            "--name",
            "-n",
            dest="project_name",
            required=True,
        )
        set_parent_project_arg(publish_samples_parser)  # args.parent_project_name

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        if args.parent_project_path is not None:
            project_path = Server.get_project_by_name_and_parent_path(logger, server, None, args.parent_project_path)
        else:
            project_path = None
        try:
            project = PublishSamplesCommand.get_project_by_name_and_parent_path(
                logger, server, args.project_name, project_path
            )
        except Exception as e:
            Errors.exit_with_error(logger, _("tabcmd.report.error.publish_samples.expected_project"), exception=e)

        try:
            server.projects.update(project, samples=True)
        except Exception as e:
            Errors.exit_with_error(logger, _("tabcmd.result.failure.publish_samples"), exception=e)



import unittest

from src.commands.project.delete_project_command import DeleteProjectCommand
from .common_setup import *

commandname = "deleteproject"


class DeleteProjectParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, DeleteProjectCommand)

    def test_delete_project(self):
        mock_args = [commandname, "project", "--parent-project-path", "p"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.parent_project_path == "p"

    def test_delete_project_required_name_none(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)



import unittest

from src.commands.extracts.encrypt_extracts_command import EncryptExtracts
from .common_setup import *

commandname = "encryptextracts"


class EncryptExtractsParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, EncryptExtracts)

    def test_encrypt_extract_parser_optional_arguments(self):
        mock_args = [commandname, "value"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.site_name == "value", args

    def test_encrypt_extract_parser_missing_all_args(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)



from src.commands.auth.session import Session
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log
from .user_data import UserCommand


class RemoveUserCommand(UserCommand):
    """
    Command to remove users from the specified group
    """

    name: str = "removeusers"
    description: str = _("removeusers.short_description")

    @staticmethod
    def define_args(remove_users_parser):
        remove_users_parser.add_argument("name", help="The group to remove users from.")
        set_users_file_arg(remove_users_parser)
        set_completeness_options(remove_users_parser)

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)

        logger.info(_("tabcmd.delete.users.from_server").format(args.users.name, args.name))

        UserCommand.act_on_users(logger, server, "removed", server.groups.remove_user, args)



import argparse
import io
import logging
from enum import IntEnum
from typing import List, Callable, Optional

import tableauserverclient as TSC

from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.localize import _


class Userdata:
    def __init__(self):
        self.name = None
        self.password = None
        self.fullname = None
        self.license_level = None
        self.admin_level = None
        self.publisher = None
        self.email = None
        self.auth = None

    def populate(self, values: List[str]) -> None:
        n_values = len(values)
        self.name = values[0]
        if n_values >= 2:
            self.password = values[1]
        if n_values >= 3:
            self.fullname = values[2]
        if n_values >= 4:
            self.license_level = values[3]
        if n_values >= 5:
            self.admin_level = values[4]
        if n_values >= 6:
            self.publisher = values[5]
        if n_values >= 7:
            self.email = values[6]
        if n_values >= 8:
            self.auth = values[7]

    def to_tsc_user(self) -> TSC.UserItem:
        site_role = UserCommand.evaluate_site_role(self.license_level, self.admin_level, self.publisher)
        if not site_role:
            raise AttributeError("Site role is required")
        user = TSC.UserItem(self.name, site_role, self.auth)
        user.email = self.email
        user.fullname = self.fullname
        return user


CHOICES: List[List[str]] = [
    [],
    [],
    [],
    ["creator", "explorer", "viewer", "unlicensed"],  # license
    ["system", "site", "none", "no"],  # admin
    ["yes", "true", "1", "no", "false", "0"],  # publisher
    [],
    [TSC.UserItem.Auth.SAML, TSC.UserItem.Auth.OpenID, TSC.UserItem.Auth.ServerDefault],  # auth
]


# username, password, display_name, license, admin_level, publishing, email, auth type
class Column(IntEnum):
    USERNAME = 0
    PASS = 1
    DISPLAY_NAME = 2
    LICENSE = 3  # aka site role
    ADMIN = 4
    PUBLISHER = 5
    EMAIL = 6
    AUTH = 7

    MAX = 7


class UserCommand(Server):
    """
    This class acts as a base class for user related group of commands
    """

    # read the file containing usernames or user details and validate each line
    # log out any errors encountered
    # returns the number of valid lines in the file
    # @param boolean strict: if true, die if any errors are found
    @staticmethod
    def validate_file_for_import(csv_file: io.TextIOWrapper, logger, detailed=False, strict=False) -> int:
        num_errors = 0
        num_valid_lines = 0
        csv_file.seek(0)  # set to start of file in case it has been read earlier
        line: str = csv_file.readline()
        while line and line != "":
            try:
                printable_line = line
                if detailed:
                    # do not print passwords
                    printable_line = line.split(",")[0]
                    UserCommand._validate_user_or_throw(line, logger)
                else:
                    logger.debug("> username - {}".format(line))
                    UserCommand._validate_username_or_throw(line)
                num_valid_lines += 1
            except Exception as exc:
                logger.info(_("importcsvsummary.error.line").format(printable_line, exc, ""))
                num_errors += 1
            line = csv_file.readline()
        if strict and num_errors > 0:
            Errors.exit_with_error(logger, _("importcsvsummary.error.too_many_errors"))
        return num_valid_lines

    # valid: username, domain/username, username@domain, domain/username@email
    @staticmethod
    def _validate_username_or_throw(username) -> None:
        if username is None or username == "" or username.strip(" ") == "":
            raise AttributeError(_("user.input.name.err.empty"))
        if username.find(" ") >= 0:
            raise AttributeError(_("tabcmd.report.error.user.no_spaces_in_username"))
        at_symbol = username.find("@")

        if at_symbol >= 0:
            username = username[:at_symbol] + "X" + username[at_symbol + 1 :]
            if username.find("@") >= 0:
                raise AttributeError(_("tabcmd.report.error.user_csv.at_char"))

    @staticmethod
    def _validate_user_or_throw(incoming, logger) -> None:
        line = list(map(str.strip, incoming.split(",")))
        logger.debug("> details - {}".format(line[0]))
        if len(line) > Column.MAX:
            raise AttributeError(_("tabcmd.report.error.user_csv.too_many_columns").format(len(line), Column.MAX))
        username = line[Column.USERNAME.value]
        UserCommand._validate_username_or_throw(username)
        for i in range(1, len(line)):
            logger.debug("column {}: {}".format(Column(i).name, line[i]))
            UserCommand._validate_item(line[i], CHOICES[i], Column(i))

    @staticmethod
    def _validate_item(item: str, possible_values: List[str], column_type) -> None:
        if item is None or item == "":
            # value can be empty for any column except user, which is checked elsewhere
            return
        if item in possible_values or possible_values == []:
            return
        raise AttributeError(_("tabcmd.report.error.generic_attribute").format(column_type, item))

    @staticmethod
    def get_users_from_file(csv_file: io.TextIOWrapper, logger=None) -> List[TSC.UserItem]:
        csv_file.seek(0)  # set to start of file in case it has been read earlier
        if logger:
            logger.debug("Reading from file {}".format(csv_file.name))
        user_list = []
        line = csv_file.readline()
        if logger:
            logger.debug("> {}".format(line))
        while line:
            user: Optional[TSC.UserItem] = UserCommand._parse_line(line)
            if user:
                user_list.append(user)
            line = csv_file.readline()
        return user_list

    @staticmethod
    def _parse_line(line: str) -> Optional[TSC.UserItem]:
        if line is None or line is False or line == "\n" or line == "":
            return None
        line = line.strip().lower()
        line_parts: List[str] = line.split(",")
        data = Userdata()
        values: List[str] = list(map(str.strip, line_parts))
        data.populate(values)
        return data.to_tsc_user()

    # https://help.tableau.com/current/server/en-us/csvguidelines.htm#settings_and_site_roles
    @staticmethod
    def evaluate_site_role(license_level, admin_level, publisher):
        if not license_level or not admin_level or not publisher:
            return "Unlicensed"
        # ignore case everywhere
        license_level = license_level.lower()
        admin_level = admin_level.lower()
        publisher = publisher.lower()
        # don't need to check publisher for system/site admin
        if admin_level == "system":
            site_role = "SiteAdministrator"
        elif admin_level == "site":
            if license_level == "creator":
                site_role = "SiteAdministratorCreator"
            elif license_level == "explorer":
                site_role = "SiteAdministratorExplorer"
            else:
                site_role = "SiteAdministratorExplorer"
        else:  # if it wasn't 'system' or 'site' then we can treat it as 'none'
            if publisher == "yes":
                if license_level == "creator":
                    site_role = "Creator"
                elif license_level == "explorer":
                    site_role = "ExplorerCanPublish"
                else:
                    site_role = "Unlicensed"  # is this the expected outcome?
            else:  # publisher == 'no':
                if license_level == "explorer" or license_level == "creator":
                    site_role = "Explorer"
                elif license_level == "viewer":
                    site_role = "Viewer"
                else:  # if license_level == 'unlicensed'
                    site_role = "Unlicensed"
        if site_role is None:
            site_role = "Unlicensed"
        return site_role

    @staticmethod
    def act_on_users(
        logger: logging.Logger, server: object, action_name: str, server_method: Callable, args: argparse.Namespace
    ) -> None:
        n_users_handled: int = 0
        number_of_errors: int = 0
        n_users_listed: int = UserCommand.validate_file_for_import(args.users, logger, strict=args.require_all_valid)
        logger.debug(_("importcsvsummary.line.processed").format(n_users_listed))

        group = None
        try:
            group = UserCommand.find_group(logger, server, args.name)
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(
                logger, _("errors.reportable.impersonation.group_not_found").format(args.name), exception=e
            )

        error_list = []
        user_obj_list: List[TSC.UserItem] = UserCommand.get_users_from_file(args.users)
        logger.debug(_("tabcmd.result.success.parsed_users").format(len(user_obj_list)))
        for user_obj in user_obj_list:
            username: str = user_obj.name or "unknown user"
            try:
                user_id: str = UserCommand.find_user_id(logger, server, username)
                logger.debug("{} user {} ({})".format(action_name, username, user_id))
            except TSC.ServerResponseError as e:
                Errors.check_common_error_codes_and_explain(logger, e)
                number_of_errors += 1
                error_list.append(e)
                logger.debug(_("tabcmd.result.failure.user").format(username))
                continue

            try:
                server_method(group, user_id)
                n_users_handled += 1
                logger.info(_("tabcmd.result.success.user_actions").format(action_name, username, group))
            except TSC.ServerResponseError as e:
                Errors.check_common_error_codes_and_explain(logger, e)
                number_of_errors += 1
                error_list.append(e)

        logger.info(_("session.monitorjob.percent_complete").format(100))
        logger.info(_("importcsvsummary.errors.count").format(number_of_errors))
        if number_of_errors > 0:
            logger.info(_("importcsvsummary.error.details").format(error_list))



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log


class CreateSiteCommand(Server):
    """
    Command to Create a site
    """

    name: str = "createsite"
    description: str = _("createsite.short_description")

    @staticmethod
    def define_args(create_site_parser):
        create_site_parser.add_argument("site_name", metavar="site-name", help=_("editsite.options.site-name"))
        set_common_site_args(create_site_parser)

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        new_site = TSC.SiteItem(
            name=args.site_name,
            content_url=args.url,
            admin_mode=args.admin_mode,
            user_quota=args.user_quota,
            storage_quota=args.storage_quota,
        )
        try:
            logger.info(_("createsite.status").format(args.site_name))
            server.sites.create(new_site)
            logger.info(_("common.output.succeeded"))
        except TSC.ServerResponseError as e:
            if Errors.is_resource_conflict(e):
                if args.continue_if_exists:
                    logger.info(_("createsite.errors.site_name_already_exists").format(args.site_name))
                    return
                else:
                    Errors.exit_with_error(
                        logger, _("createsite.errors.site_name_already_exists").format(args.site_name)
                    )

            Errors.exit_with_error(logger, _("publish.errors.unexpected_server_response"), e)



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.localize import _
from src.execution.logger_config import log


class DeleteGroupCommand(Server):
    """
    This command deletes the specified group from the server
    """

    name: str = "deletegroup"
    description: str = _("deletegroup.short_description")

    @staticmethod
    def define_args(delete_group_parser):
        delete_group_parser.add_argument("name")

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        try:
            logger.info(_("tabcmd.find.group").format(args.name))
            group_id = Server.find_group_id(logger, server, args.name)
            logger.info(_("deletegroup.status").format(group_id))
            server.groups.delete(group_id)
            logger.info(_("tabcmd.result.succeeded"))
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, "tabcmd.result.failed.delete.group", e)



import unittest

from src.commands.datasources_and_workbooks.export_command import ExportCommand
from .common_setup import *

commandname = "export"


class ExportParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, ExportCommand)

    def test_export_parser_file_type_pdf(self):
        mock_args = mock_args = ["export", "helloworld", "--pdf"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.pdf is True, args
        assert args.url == "helloworld", args

    def test_export_parser_missing_all_args(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log
from .datasources_and_workbooks_command import DatasourcesAndWorkbooks


class PublishCommand(DatasourcesAndWorkbooks):
    """
    This command publishes the specified workbook (.twb(x)), data source
    (.tds(x)), or extract (.hyper) to Tableau Server.
    """

    name: str = "publish"
    description: str = _("publish.description")

    @staticmethod
    def define_args(publish_parser):
        publish_parser.add_argument(
            "filename",
            metavar="filename.twbx|tdsx|hyper",
            # this is not actually a File type because we just pass the path to tsc
        )
        set_publish_args(publish_parser)
        set_project_r_arg(publish_parser)
        set_parent_project_arg(publish_parser)

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)

        if args.project_name:
            try:
                project_id = Server.get_project_by_name_and_parent_path(
                    logger, server, args.project_name, args.parent_project_path
                )
            except Exception as exc:
                Errors.exit_with_error(logger, _("publish.errors.server_resource_not_found"), exc)
        else:
            project_id = ""
            args.project_name = "default"
            args.parent_project_path = ""

        publish_mode = PublishCommand.get_publish_mode(args)

        source = PublishCommand.get_filename_extension_if_tableau_type(logger, args.filename)
        logger.info(_("publish.status").format(args.filename))
        if source in ["twbx", "twb"]:
            new_workbook = TSC.WorkbookItem(project_id, name=args.name, show_tabs=args.tabbed)
            try:
                new_workbook = server.workbooks.publish(new_workbook, args.filename, publish_mode)
            except IOError as ioe:
                Errors.exit_with_error(logger, ioe)
            logger.info(_("publish.success") + "\n{}".format(new_workbook.webpage_url))

        elif source in ["tds", "tdsx", "hyper"]:
            new_datasource = TSC.DatasourceItem(project_id, name=args.name)
            try:
                new_datasource = server.datasources.publish(new_datasource, args.filename, publish_mode)
            except IOError as ioe:
                Errors.exit_with_error(logger, exc)
            logger.info(_("publish.success") + "\n{}".format(new_datasource.webpage_url))

    @staticmethod
    def get_publish_mode(args):
        if args.overwrite:
            publish_mode = TSC.Server.PublishMode.Overwrite
        else:
            publish_mode = TSC.Server.PublishMode.CreateNew
        return publish_mode



import sys


class HelpCommand:
    """
    Command to show user help options
    """

    name: str = "help"
    description: str = "Show Help and exit"

    @staticmethod
    def define_args(parser):
        # takes no args
        pass

    @staticmethod
    def run_command(args):
        description = (
            "tabcmd - Tableau Server Command Line Utility 2.0 \n \n"
            "tabcmd help             -- Help for tabcmd commands \n"
            "tabcmd help <a command> -- Show Help for a specific command\n"
            "tabcmd help commands    -- List all available commands\n\n"
        )
        sys.stdout.write(description)



import getpass
import json
import os

import requests
import tableauserverclient as TSC
from urllib3.exceptions import InsecureRequestWarning

from src.commands.constants import Errors
from src.execution.localize import _
from src.execution.logger_config import log


class Session:
    """
    Session class handles all authentication related work
    """

    TOKEN_CRED_TYPE = "token"
    PASSWORD_CRED_TYPE = "password"

    TOKEN_CRED_TYPE = "token"
    PASSWORD_CRED_TYPE = "password"

    def __init__(self):
        self.username = None
        # we don't store the password
        self.user_id = None
        self.auth_token = None
        self.token_name = None
        self.token = None
        self.password_file = None
        self.site_name = None  # The site name, e.g 'alpodev'
        self.site_id = None  # The site id, e.g 'abcd-1234-1234-1244-1234'
        self.server_url = None
        self.last_command = None  # for when we have to renew the session then re-try
        self.last_login_using = None

        self.no_prompt = False
        self.certificate = None
        self.no_certcheck = False
        self.no_proxy = True
        self.proxy = None
        self.timeout = None

        self.logging_level = "info"
        self.logger = log(__class__.__name__, self.logging_level)
        self._read_from_json()
        self.tableau_server = None  # this one is an object that doesn't get persisted in the file

    # called before we connect to the server
    # generally, we don't want to overwrite stored data with nulls
    def _update_session_data(self, args):
        # user id and site id are never passed in as args
        # last_login_using and tableau_server are internal data
        # self.command = args.???
        # TODO: if server/username/token are changed, clear others
        self.username = args.username or self.username
        self.site_name = args.site_name or self.site_name or ""
        if self.site_name == "default":
            self.site_name = ""
        self.server_url = args.server or self.server_url or "http://localhost"
        self.logging_level = args.logging_level or self.logging_level
        self.password_file = args.password_file
        self.token_name = args.token_name or self.token_name
        self.token = args.token or self.token

        self.no_prompt = args.no_prompt or self.no_prompt
        self.certificate = args.certificate or self.certificate
        self.no_certcheck = args.no_certcheck or self.no_certcheck
        self.no_proxy = args.no_proxy or self.no_proxy
        self.proxy = args.proxy or self.proxy
        self.timeout = args.timeout or self.timeout

    @staticmethod
    def _read_password_from_file(filename):
        credential = None
        with open(str(filename), "r") as file_contents:
            reader = file_contents.readlines()
            for row in reader:
                credential = row
            return credential

    def _allow_prompt(self):
        try:
            return not self.no_prompt
        except Exception:
            return True

    def _create_new_credential(self, password, credential_type):
        if password is None:
            if self.password_file:
                password = Session._read_password_from_file(self.password_file)
            elif self._allow_prompt():
                password = getpass.getpass(_("session.password"))
            else:
                Errors.exit_with_error(self.logger, _("session.errors.script_no_password"))

        if credential_type == Session.PASSWORD_CRED_TYPE and self.username and password:
            credentials = TSC.TableauAuth(self.username, password, site_id=self.site_name)
            self.last_login_using = "username"
            return credentials
        elif credential_type == Session.TOKEN_CRED_TYPE and self.token_name:
            credentials = self._create_new_token_credential()
            return credentials
        else:
            Errors.exit_with_error(self.logger, "Couldn't find credentials")

    def _create_new_token_credential(self):
        if self.token:
            token = self.token
        elif self.password_file:
            token = Session._read_password_from_file(self.password_file)
        elif self._allow_prompt():
            token = getpass.getpass("Token:")
        else:
            Errors.exit_with_error(self.logger, _("session.errors.missing_arguments").format("token"))

        if self.token_name and token:
            credentials = TSC.PersonalAccessTokenAuth(self.token_name, token, site_id=self.site_name)
            self.last_login_using = "token"
            return credentials
        else:
            Errors.exit_with_error(self.logger, _("session.errors.missing_arguments").format("token name"))

    def _set_connection_options(self):
        # args still to be handled here:
        # proxy, --no-proxy,
        # cert
        # timeout
        http_options = {}
        if self.no_certcheck:
            http_options = {"verify": False}
            requests.packages.urllib3.disable_warnings(category=InsecureRequestWarning)
        tableau_server = TSC.Server(self.server_url, use_server_version=True, http_options=http_options)

        return tableau_server

    def _create_new_connection(self):
        self.logger.info(_("session.new_session"))
        self.tableau_server = self._set_connection_options()
        self._print_server_info()
        self.logger.info(_("session.connecting"))
        self.tableau_server.use_server_version()  # this will attempt to contact the server

    def _read_existing_state(self):
        if self._check_json():
            self._read_from_json()

    def _print_server_info(self):
        self.logger.info("=====   Server: {}".format(self.server_url))
        if self.username:
            self.logger.info("=====   Username: {}".format(self.username))
        else:
            self.logger.info("=====   Token Name: {}".format(self.token_name))
        self.logger.info(_("dataconnections.classes.tableau_server_site") + ": {}".format(self.site_name))

    def _validate_existing_signin(self):
        self.logger.info(_("session.continuing_session"))
        self.tableau_server = self._set_connection_options()
        try:
            if self.tableau_server and self.tableau_server.is_signed_in():
                response = self.tableau_server.users.get_by_id(self.user_id)
                self.logger.debug(response)
                if response.status_code.startswith("200"):
                    return self.tableau_server
        except TSC.ServerResponseError as e:
            self.logger.info(_("publish.errors.unexpected_server_response"), e)
        except Exception as e:
            self.logger.info(_("errors.internal_error.request.message"), e)
        return None

    def _sign_in(self, tableau_auth):
        self.logger.debug(_("session.login") + self.server_url)
        self.logger.debug(_("listsites.output").format("", self.username or self.token_name, self.site_name))
        try:
            self.tableau_server.auth.sign_in(tableau_auth)  # it's the same call for token or user-pass
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(self.logger, exception=e)
        try:
            self.site_id = self.tableau_server.site_id
            self.user_id = self.tableau_server.user_id
            self.auth_token = self.tableau_server._auth_token
            if not self.username:
                self.username = self.tableau_server.users.get_by_id(self.user_id).name
            self.logger.debug("Signed into {0}{1} as {2}".format(self.server_url, self.site_name, self.username))
            self.logger.info(_("common.output.succeeded"))
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(self.logger, _("publish.errors.unexpected_server_response"), e)

        return self.tableau_server

    def _get_saved_credentials(self):
        if self.last_login_using == "username":
            credentials = self._create_new_credential(None, Session.PASSWORD_CRED_TYPE)
        elif self.last_login_using == "token":
            credentials = self._create_new_token_credential()
        else:
            return None
        if credentials:
            self.logger.info(_("session.options.password-file"))
        return credentials

    # external entry point:
    def create_session(self, args):
        signed_in_object = None
        # pull out cached info from json, then overwrite with new args if available
        self._read_existing_state()
        self._update_session_data(args)
        self.logging_level = args.logging_level or self.logging_level

        credentials = None
        if args.password:
            self._end_session()
            # we don't save the password anywhere, so we pass it along directly
            credentials = self._create_new_credential(args.password, Session.PASSWORD_CRED_TYPE)
        elif args.password_file:
            self._end_session()
            if args.username:
                credentials = self._create_new_credential(args.password, Session.PASSWORD_CRED_TYPE)
            else:
                credentials = self._create_new_credential(args.password, Session.TOKEN_CRED_TYPE)
        elif args.token:
            self._end_session()
            credentials = self._create_new_token_credential()
        else:  # no login arguments given - look for saved info
            # maybe we're already signed in!
            if self.tableau_server:
                signed_in_object = self._validate_existing_signin()
            self.logger.debug(signed_in_object)
            # or maybe we at least have the credentials saved
            if not signed_in_object:
                credentials = self._get_saved_credentials()

        if credentials and not signed_in_object:
            # logging in, not using an existing session
            self._create_new_connection()
            signed_in_object = self._sign_in(credentials)

        if not signed_in_object:
            missing_var = _("editdomain.errors.requires_nickname_name").format("username", "token")
            Errors.exit_with_error(self.logger, _("session.errors.missing_arguments").format(missing_var))
        if args.no_cookie:
            self._remove_json()
        else:
            self._save_session_to_json()
        return signed_in_object

    def end_session_and_clear_data(self):
        self._end_session()
        self.logger.info(_("session.logout"))
        self._clear_data()

    def _end_session(self):
        if self.tableau_server:
            self.tableau_server.auth.sign_out()
            self.tableau_server = None

    def _clear_data(self):
        self._remove_json()
        self.username = None
        self.user_id = None
        self.auth_token = None
        self.token_name = None
        self.token = None
        self.site_name = None
        self.site_id = None
        self.server = None
        self.last_login_using = None
        self.password_file = None

        self.last_command = None
        self.tableau_server = None

        self.certificate = None
        self.no_certcheck = None
        self.no_proxy = None
        self.proxy = None
        self.timeout = None

    # json file functions ----------------------------------------------------
    def _get_file_path(self):
        home_path = os.path.expanduser("~")
        file_path = os.path.join(home_path, "tableau_auth.json")
        return file_path

    def _read_from_json(self):
        if not self._check_json():
            return
        file_path = self._get_file_path()
        data = {}
        with open(str(file_path), "r") as file_contents:
            data = json.load(file_contents)
        try:
            for auth in data["tableau_auth"]:
                self.auth_token = auth["auth_token"]
                self.server_url = auth["server"]
                self.site_name = auth["site_name"]
                self.site_id = auth["site_id"]
                self.username = auth["username"]
                self.user_id = auth["user_id"]
                self.token_name = auth["personal_access_token_name"]
                self.token = auth["personal_access_token"]
                self.last_login_using = auth["last_login_using"]
                self.password_file = auth["password_file"]
                self.no_prompt = auth["no_prompt"]
                self.no_certcheck = auth["no_certcheck"]
                self.certificate = auth["certificate"]
                self.no_proxy = auth["no_proxy"]
                self.proxy = auth["proxy"]
                self.timeout = auth["timeout"]
        except KeyError as e:
            self.logger.debug(_("sessionoptions.errors.bad_password_file"), e)
            self._remove_json()
        except Exception as any_error:
            self.logger.info(_("session.new_session"))
            self._remove_json()

    def _check_json(self):
        home_path = os.path.expanduser("~")
        file_path = os.path.join(home_path, "tableau_auth.json")
        return os.path.exists(file_path)

    def _save_session_to_json(self):
        data = self._serialize_for_save()
        self._save_file(data)

    def _save_file(self, data):
        file_path = self._get_file_path()
        with open(str(file_path), "w") as f:
            json.dump(data, f)

    def _serialize_for_save(self):
        data = {"tableau_auth": []}
        data["tableau_auth"].append(
            {
                "auth_token": self.auth_token,
                "server": self.server_url,
                "username": self.username,
                "user_id": self.user_id,
                "site_name": self.site_name,
                "site_id": self.site_id,
                "personal_access_token_name": self.token_name,
                "personal_access_token": self.token,
                "last_login_using": self.last_login_using,
                "password_file": self.password_file,
                "no_prompt": self.no_prompt,
                "no_certcheck": self.no_certcheck,
                "certificate": self.certificate,
                "no_proxy": self.no_proxy,
                "proxy": self.proxy,
                "timeout": self.timeout,
            }
        )
        return data

    def _remove_json(self):
        file_path = self._get_file_path()
        self._save_file({})
        if os.path.exists(file_path):
            os.remove(file_path)



import os

import tableauserverclient as TSC

from src.commands.constants import Errors


class Server:
    # syntactic sugar for specific content types
    @staticmethod
    def get_workbook_item(logger, server, workbook_name, container=None):
        try:
            return Server.get_items_by_name(logger, server.workbooks, workbook_name, container=None)[0]
        except Exception as e:
            Errors.exit_with_error(logger, exception=e)

    @staticmethod
    def get_workbook_id(logger, server, workbook_name, container=None):
        return Server.get_workbook_item(logger, server, workbook_name, container=None).id

    @staticmethod
    def get_data_source_item(logger, server, data_source_name, container=None):
        try:
            return Server.get_items_by_name(logger, server.datasources, data_source_name, container=None)[0]
        except Exception as e:
            Errors.exit_with_error(logger, exception=e)

    @staticmethod
    def get_data_source_id(logger, server, data_source_name, container=None):
        return Server.get_data_source_item(logger, server, data_source_name, container=None).id

    @staticmethod
    def find_group(logger, server, group_name):
        try:
            return Server.get_items_by_name(logger, server.groups, group_name)[0]
        except Exception as e:
            Errors.exit_with_error(logger, exception=e)

    @staticmethod
    def find_group_id(logger, server, group_name):
        return Server.find_group(logger, server, group_name)[0].id

    @staticmethod
    def find_user_id(logger, server, username):
        try:
            return Server.get_items_by_name(logger, server.users, username)[0].id
        except Exception as e:
            Errors.exit_with_error(logger, exception=e)

    @staticmethod
    def get_items_by_name(logger, item_endpoint, item_name, container=None):
        logger.debug(_("export.status").format(item_name))
        req_option = TSC.RequestOptions()
        req_option.filter.add(TSC.Filter(TSC.RequestOptions.Field.Name, TSC.RequestOptions.Operator.Equals, item_name))
        if container:
            logger.debug("Searching in project {}".format(container))
            req_option.filter.add(
                TSC.Filter(TSC.RequestOptions.Field.ParentProjectId, TSC.RequestOptions.Operator.Equals, container)
            )
        all_items, pagination_item = item_endpoint.get(req_option)
        if all_items is None or all_items == []:
            raise ValueError(_("publish.errors.server_resource_not_found"))
        if len(all_items) > 1:
            logger.debug("{}+ items of this name were found. Returning first page.".format(len(all_items)))
        return all_items

    # Get site by name or get currently logged in site
    @staticmethod
    def get_site_for_command_or_throw(logger, server, args):
        if args.site_name:
            try:
                site_item = Server.get_items_by_name(logger, server.sites, args.site_name)[0]
            except Exception as e:
                Errors.exit_with_error(logger, exception=e)
        else:
            logger.debug("Use logged in site")
            # site_item = server.sites.get_by_id(server.site_id)
            if not site_item:
                raise ResourceWarning("Could not get site from server")
        return site_item

    @staticmethod
    def get_filename_extension_if_tableau_type(logger, filename):
        logger.debug("Filename given: {}".format(filename))
        source_file, source_type = os.path.splitext(filename)  # returns .ext
        source_type = source_type.lstrip(".")
        possible_types = ["twbx", "twb", "tdsx", "tds", "hyper"]
        if source_type and source_type in possible_types:
            return source_type
        else:
            raise ValueError(
                "Filename `{0}` does not have an appropriate file extension: found `{1}`.".format(filename, source_type)
            )

    @staticmethod
    def get_project_by_name_and_parent_path(logger, server, project_name, parent_path):
        logger.debug(_("content_type.project") + ":{0}, {1}".format(parent_path, project_name))
        project_tree = Server._parse_project_path_to_list(parent_path)
        if not project_name:
            project = Server._get_parent_project_from_tree(logger, server, project_tree)
        else:
            parent = Server._get_parent_project_from_tree(logger, server, project_tree)
            project = Server._get_project_by_name_and_parent(logger, server, project_name, parent)
        if not project:
            Errors.exit_with_error(logger, message=_("publish.errors.server_resource_not_found"))
        return project

    @staticmethod
    def _parse_project_path_to_list(project_path):
        if project_path is None:
            return []
        return project_path.split("/")

    @staticmethod
    def _get_project_by_name_and_parent(logger, server, project_name, parent):
        # logger.debug("get by name and parent: {0}, {1}".format(project_name, parent))
        # get by name to narrow down the list
        projects = Server.get_items_by_name(logger, server.projects, project_name)
        if parent is not None:
            parent_id = parent.id
            for project in projects:
                if project.parent_id == parent_id:
                    return project
        return projects[0]

    @staticmethod
    def _get_parent_project_from_tree(logger, server, hierarchy):
        # logger.debug("get parent project from tree: {0}".format(hierarchy))
        tree_height = len(hierarchy)
        if tree_height == 0:
            return None
        elif tree_height == 1:
            return Server._get_project_by_name_and_parent(logger, server, hierarchy[0], None)
        else:
            name = hierarchy.pop(tree_height - 1)
            return Server._get_project_by_name_and_parent(
                logger, server, name, Server._get_parent_project_from_tree(logger, server, hierarchy)
            )



class Commands:
    def __init__(self, args):
        self.logger = None
        self.username = args.username
        self.password = args.password
        self.server = args.server
        self.site = args.site
        self.token_name = args.token_name
        self.personal_token = args.token
        self.logging_level = args.logging_level



import argparse
from . import _version
from .localize import _


class ParentParser:
    # Ref https://docs.python.org/3/library/argparse.html
    """Parser that will be inherited by all commands. Contains
    authentication and logging level setting"""

    def __init__(self):
        self.global_options = self.parent_parser_with_global_options()
        self.root = argparse.ArgumentParser(parents=[self.global_options])
        # https://stackoverflow.com/questions/7498595/python-argparse-add-argument-to-multiple-subparsers
        self.subparsers = self.root.add_subparsers()

    def get_root_parser(self):
        return self.root

    def include(self, command):
        additional_parser = self.subparsers.add_parser(
            command.name, help=command.description, parents=[self.global_options]
        )
        # This line is where we actually set each parser to call the correct command
        additional_parser.set_defaults(func=command)
        command.define_args(additional_parser)
        return additional_parser

    # ordered alphabetically by short option - this is reflected directly in help output
    def parent_parser_with_global_options(self):
        parser = argparse.ArgumentParser(usage=argparse.SUPPRESS, add_help=False)

        certificates = parser.add_mutually_exclusive_group()
        certificates.add_argument(
            "-c",
            "--use-certificate",
            dest="certificate",
            default=None,
            metavar="",
            help=_("session.options.use-certificate"),
        )
        certificates.add_argument(
            "--no-certcheck",
            action="store_true",
            help=_("session.options.no-certcheck"),
        )

        parser.add_argument(
            "--continue-if-exists",
            action="store_false",
            help="Treat resource conflicts as item creation success e.g project already exists",
        )

        parser.add_argument("--no-cookie", action="store_true", help=_("session.options.no-cookie"))

        parser.add_argument(
            "-l",
            "--logging-level",
            choices=["DEBUG", "INFO", "ERROR"],
            default="info",
            metavar="",
            help="Use the specified logging level. The default level is INFO.",
        )

        parser.add_argument("--no-prompt", action="store_true", help=_("session.options.no-prompt"))

        auth_options = parser.add_mutually_exclusive_group()
        auth_options.add_argument(
            "-tn",
            "--token-name",
            default=None,
            metavar="<TOKEN NAME>",
            help="The name of the Tableau Server Personal Access Token. If using a token to sign in,\
                  this is required at least once to begin session.",
        )
        auth_options.add_argument(
            "-u", "--username", default=None, metavar="<USER>", help=_("session.options.username")
        )

        secret_values = parser.add_mutually_exclusive_group()
        secret_values.add_argument(
            "-to",
            "--token",
            default=None,
            metavar="<TOKEN VALUE>",
            help="Use the specified Tableau Server Personal Access Token. Requires --token-name to be set.",
        )
        secret_values.add_argument(
            "-p", "--password", default=None, metavar="<PASSWORD>", help=_("session.options.password")
        )
        secret_values.add_argument(
            "--password-file", default=None, metavar="<FILE>", help=_("session.options.password-file")
        )

        proxy_group = parser.add_mutually_exclusive_group()
        proxy_group.add_argument(
            "-x", "--proxy", dest="proxy", default=None, metavar="<HOST:PORT>", help=_("session.options.proxy")
        )
        proxy_group.add_argument(
            "--no-proxy",
            action="store_false",
            help=_("session.options.no-proxy"),
        )

        parser.add_argument(
            "-s",
            "--server",
            default=None,  # default is handled in Session class
            metavar="<URL>",
            help=_("session.options.server"),
        )
        parser.add_argument(
            "-t", "--site", default="", dest="site_name", metavar="SITEID", help=_("session.options.site")
        )

        parser.add_argument(
            "--timeout",
            default=None,  # default is handled in Session class
            metavar="<SECONDS>",  # can't use -t, it's already used for --site
            help=_("session.options.timeout"),
        )

        parser.add_argument(
            "-v",
            "--version",
            action="version",
            version="tabcmd.exe - Tableau Server Command Line Utility v" + _version.version + "\n \n",
            help="Show version information and exit.",
        )

        # TODO get the list of choices dynamically?
        parser.add_argument(
            "--language",
            choices=["de", "en", "es", "fr", "it", "ja", "ko", "pt", "sv", "zh"],
            help="Set the language to use. Exported data will be returned in this lang/locale."
            "If not set, the client will use your computer locale, and the server will use your user account locale",
        )

        parser.add_argument(
            "--country",
            choices=["de", "en", "es", "fr", "it", "ja", "ko", "pt", "sv", "zh"],
            help=_("export.options.country"),
        )
        return parser



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log


class CreateProjectCommand(Server):
    """
    Command to create a project
    """

    name: str = "createproject"
    description: str = _("createproject.short_description")

    @staticmethod
    def define_args(create_project_parser):
        create_project_parser.add_argument(
            "--name", "-n", dest="project_name", required=True, help=_("createproject.options.name")
        )
        set_parent_project_arg(create_project_parser)
        set_description_arg(create_project_parser)

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        parent_id = None
        readable_name = args.project_name
        if args.parent_project_path:
            try:
                logger.info(_("tabcmd.find.parent_project").format(args.parent_project_path))
                parent = Server.get_project_by_name_and_parent_path(logger, server, None, args.parent_project_path)
            except TSC.ServerResponseError as exc:
                Errors.exit_with_error(logger, _("publish.errors.server_resource_not_found"), exc)
            readable_name = "{0}/{1}".format(args.parent_project_path, args.project_name)
            parent_id = parent.id
            logger.debug("parent project = `{0}`, id = {1}".format(args.parent_project_path, parent_id))
        logger.info(_("createproject.status").format(readable_name))
        new_project = TSC.ProjectItem(args.project_name, args.description, None, parent_id)
        try:
            project_item = server.projects.create(new_project)
            logger.info(_("common.output.succeeded"))
            return project_item
        except TSC.ServerResponseError as e:
            if Errors.is_resource_conflict(e):
                if args.continue_if_exists:
                    logger.info(_("tabcmd.result.already_exists").format(args.project_name))
                    return
                else:
                    Errors.exit_with_error(logger, _("tabcmd.result.already_exists").format(args.project_name))
            Errors.exit_with_error(logger, _("publish.errors.unexpected_server_response"), e)



import unittest

from src.commands.datasources_and_workbooks.runschedule_command import RunSchedule
from .common_setup import *

commandname = "runschedule"


class RunScheduleParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, RunSchedule)

    def test_runschedule_parser_required_name(self):
        mock_args = [commandname, "schedulename"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.schedule == "schedulename"

    def test_runschedule_parser_missing_all_args(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            self.parser_under_test.parse_args(mock_args)



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log
from .datasources_and_workbooks_command import DatasourcesAndWorkbooks


class DeleteCommand(DatasourcesAndWorkbooks):
    """
    Command to delete the specified workbook or data source from the server.
    """

    name: str = "delete"
    description: str = _("delete.short_description")

    located_workbook = None
    located_datasource = None

    @staticmethod
    def define_args(delete_parser):
        delete_parser.add_argument("name", help=_("content_type.workbook") + _("content_type.datasource"))
        set_project_r_arg(delete_parser)
        set_parent_project_arg(delete_parser)

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)

        logger.info(_("delete.status").format(args.name, ""))

        error = None
        try:
            item_to_delete = DeleteCommand.get_workbook_item(logger, server, args.name)
            item_type = "workbook"
        except TSC.ServerResponseError as workbook_error:
            error = workbook_error
        try:
            item_to_delete = DeleteCommand.get_data_source_item(logger, server, args.name)
            item_type = "datasource"
        except TSC.ServerResponseError as ds_error:
            error = ds_error
        if not item_type:
            logger.debug(error)
            Errors.exit_with_error(logger, _("delete.errors.requires_workbook_datasource"))

        try:
            if item_type == "workbook":
                server.workbooks.delete(item_to_delete.id)
            else:
                server.datasources.delete(item_to_delete.id)
            logger.info(_("common.output.succeeded"))
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, "Error deleting from server", e)



import polling2  # type: ignore
import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log


class RefreshExtracts(Server):

    name: str = "refreshextracts"
    description: str = _("refreshextracts.short_description")

    @staticmethod
    def define_args(refresh_extract_parser):
        possible_targets = set_ds_xor_wb_args(refresh_extract_parser)
        possible_targets.add_argument(
            "--url",
            help=_("createextracts.options.url"),
        )
        set_incremental_options(refresh_extract_parser)
        set_calculations_options(refresh_extract_parser)
        set_project_arg(refresh_extract_parser)
        set_parent_project_arg(refresh_extract_parser)

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        refresh_action = "refresh"

        if args.addcalculations or args.removecalculations:
            logger.warning(
                "Data Acceleration tasks are deprecated and this parameter has no effect."
                "It will be removed in a future update."
            )

        # are these two mandatory? mutually exclusive?
        # docs: the REST method always runs a full refresh even if the refresh type is set to incremental.
        if args.incremental:  # docs: run the incremental refresh
            logger.warn("Incremental refresh is not yet available through the new tabcmd")
        # if args.synchronous:  # docs: run a full refresh and poll until it completes
        # else:  run a full refresh but don't poll for completion

        container = None
        if args.project_name:
            try:
                container = Server.get_project_by_name_and_parent_path(
                    logger, server, args.project_name, args.parent_project_path
                )
            except Exception as ex:
                logger.warning(
                    "Could not find project {}/{}. Continuing without.".format(
                        args.parent_project_path, args.project_name
                    )
                )
        job = None
        try:
            # TODO: use the container in the search
            if args.datasource:
                logger.debug(_("export.status").format(args.datasource))
                datasource_id = Server.get_data_source_id(logger, server, args.datasource, container)
                logger.info(_("refreshextracts.status_refreshed").format(_("content_type.datasource"), args.datasource))
                job: TSC.JobItem = server.datasources.refresh(datasource_id)

            elif args.workbook:
                logger.debug(_("export.status").format(args.workbook))
                workbook_id = Server.get_workbook_id(logger, server, args.workbook, container)
                logger.info(_("refreshextracts.status_refreshed").format(_("content_type.workbook"), args.workbook))
                job: TSC.JobItem = server.workbooks.refresh(workbook_id)

            elif args.url:
                logger.error("URL not yet implemented")

        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, _("refreshextracts.errors.error"), e)

        logger.info(_("common.output.job_queued_success"))

        if args.synchronous:
            # maintains a live connection to the server while the refresh operation is underway, polling every second
            # until the background job is done.   <job id="JOB_ID" mode="MODE" type="RefreshExtract" />
            logger.info("Waiting for refresh job to begin ....")

            try:
                polling2.poll(lambda: logger.info(".") and job.started_at is not None, step=1, timeout=args.timeout)
            except polling2.TimeoutException as te:
                Errors.exit_with_error(logger, _("messages.timeout_error.summary"))
            logger.info("Job started at {}".format(job.started_at))

            try:
                polling2.poll(
                    lambda: logger.info("{}".format(job.progress)) and job.finish_code != -1,
                    step=1,
                    timeout=args.timeout,
                )
                logger.info("Job completed at {}".format(job.completed_at))
            except polling2.TimeoutException as te:
                Errors.exit_with_error(logger, _("messages.timeout_error.summary"))

        else:
            logger.info(_("common.output.job_queued_success"))
            logger.debug("Extract refresh started with JobID: {0}".format(job.id))



import unittest

from src.commands.project.create_project_command import CreateProjectCommand
from .common_setup import *

commandname = "createproject"


class CreateProjectParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, CreateProjectCommand)

    def test_create_project_parser_optional_arguments(self):
        mock_args = [
            commandname,
            "--name",
            "testproject",
            "--parent-project-path",
            "abcdef",
            "--description",
            "desc",
        ]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.project_name == "testproject"
        assert args.parent_project_path == "abcdef"

    def test_create_project_parser_required_arguments_name(self):
        mock_args = [
            commandname,
            "-n",
            "project-name",
            "--parent-project-path",
            "abcdef",
            "--description",
            "desc",
        ]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.project_name == "project-name"
        assert args.parent_project_path == "abcdef"

    def test_create_project_parser_required_arguments_missing_name(self):
        mock_args = [
            commandname,
            "--parent-project-path",
            "abcdef",
            "--description",
            "desc",
        ]
        with self.assertRaises(SystemExit):
            self.parser_under_test.parse_args(mock_args)

    def test_create_project_parser_optional_arguments_missing_project_path(self):
        mock_args = [
            commandname,
            "-n",
            "project-name",
            "--parent-project-path",
            "--description",
            "desc",
        ]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)



import unittest
from unittest import mock

from src.commands.user.create_site_users import CreateSiteUsersCommand
from .common_setup import *

commandname = "createsiteusers"


class CreateSiteUsersParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, CreateSiteUsersCommand)

    def test_create_site_users_parser_users_file(self):
        with mock.patch("builtins.open", mock.mock_open(read_data="test")) as open_file:
            mock_args = [commandname, "users.csv"]
            args = self.parser_under_test.parse_args(mock_args)
            open_file.assert_called_with("users.csv", "r", -1, "UTF-8", None)

    def test_create_site_user_parser_missing_arguments(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_create_site_user_parser_role(self):
        with mock.patch("builtins.open", mock.mock_open(read_data="test")):
            mock_args = [commandname, "users.csv", "--site", "site-name"]
            args = self.parser_under_test.parse_args(mock_args)
            assert args.site_name == "site-name", args



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.execution.localize import _
from src.execution.logger_config import log
from .datasources_and_workbooks_command import DatasourcesAndWorkbooks


class ExportCommand(DatasourcesAndWorkbooks):

    name: str = "export"
    description: str = _("export.short_description")

    @staticmethod
    def define_args(export_parser):
        export_parser.add_argument("url", help="url of the workbook or view to export")
        export_parser_group = export_parser.add_mutually_exclusive_group(required=True)
        export_parser_group.add_argument("--pdf", action="store_true", help=_("export.options.pdf"))
        export_parser_group.add_argument("--fullpdf", action="store_true", help=_("export.options.fullpdf"))
        export_parser_group.add_argument("--png", action="store_true", help=_("export.options.png"))
        export_parser_group.add_argument("--csv", action="store_true", help=_("export.options.csv"))

        export_parser.add_argument(
            "--pagelayout",
            choices=["landscape", "portrait"],
            default="landscape",
            help="page orientation (landscape or portrait) of the exported PDF",
        )
        export_parser.add_argument("--pagesize", default="letter", help="Set the page size of the exported PDF")
        export_parser.add_argument("--width", default=800, help="Set the width in pixels. Default is 800 px")
        export_parser.add_argument("--filename", "-f", help="filename to store the exported data")
        export_parser.add_argument("--height", default=600, help=_("export.options.height"))
        export_parser.add_argument(
            "--filter",
            "-vf",
            metavar="COLUMN:VALUE",
            help="View filter to apply to the view",
        )

    # TODO: ARGUMENT --COMPLETE

    @staticmethod
    def get_content_url_for_workbook(url):
        # check the size of list
        separated_list = url.split("/")
        reversed_list = separated_list[::-1]
        return reversed_list[1]

    @staticmethod
    def get_content_url_for_view(url):
        # check the size of list
        separated_list = url.split("/")
        if len(separated_list) > 2:
            print("error")
        workbook_name = separated_list[0]
        view_name = separated_list[1]
        return DatasourcesAndWorkbooks.get_view_url_from_names(workbook_name, view_name)

    """
    Command to Export a view_name or workbook from Tableau Server and save
    it to a file. This command can also export just the data used for a view_name
    """

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        view_content_url, wb_content_url = ExportCommand.parse_export_url_to_workbook_and_view(logger, args.url)
        logger.debug([view_content_url, wb_content_url])
        if not view_content_url and not wb_content_url:
            Errors.exit_with_error(logger, _("export.errors.requires_workbook_view_param").format(ExportCommand))

        try:

            if args.fullpdf:  # it's a workbook
                workbook_item = ExportCommand.get_wb_by_content_url(logger, server, wb_content_url)
                output = ExportCommand.download_wb_pdf(server, workbook_item)
                default_filename = "{}.pdf".format(workbook_item.name)

            elif args.pdf or args.png or args.csv:  # it's a view
                view_item = ExportCommand.get_view_by_content_url(logger, server, view_content_url)

                if args.pdf:
                    output = ExportCommand.download_view_pdf(server, view_item)
                    default_filename = "{}.pdf".format(view_item.name)
                elif args.csv:
                    output = ExportCommand.download_csv(server, view_item)
                    default_filename = "{}.csv".format(view_item.name)
                elif args.png:
                    output = ExportCommand.download_png(server, view_item)
                    default_filename = "{}.png".format(view_item.name)

        except Exception as e:
            Errors.exit_with_error(logger, _("publish.errors.unexpected_server_response").format(""), e)

        try:
            save_name = args.filename or default_filename
            ExportCommand.save_to_file(logger, output, save_name)

        except Exception as e:
            Errors.exit_with_error(logger, "Error saving to file", e)

    @staticmethod
    def download_wb_pdf(server, workbook_item):
        pdf = TSC.PDFRequestOptions(maxage=1)
        server.workbooks.populate_pdf(workbook_item, pdf)
        return workbook_item.pdf

    @staticmethod
    def download_view_pdf(server, view_item):
        pdf = TSC.PDFRequestOptions(maxage=1)
        server.views.populate_pdf(view_item, pdf)
        return view_item.pdf

    @staticmethod
    def download_csv(server, view_item):
        csv = TSC.CSVRequestOptions(maxage=1)
        server.views.populate_csv(view_item, csv)
        return view_item.csv

    @staticmethod
    def download_png(server, view_item):
        req_option_image = TSC.ImageRequestOptions(maxage=1)
        server.views.populate_image(view_item, req_option_image)
        return view_item.png

    @staticmethod
    def parse_export_url_to_workbook_and_view(logger, url):
        logger.info(_("export.status").format(url))
        if " " in url:
            Errors.exit_with_error(logger, _("export.errors.white_space_workbook_view"))
        # input should be workbook_name/view_name
        if not url.find("/"):
            return None, None
        name_parts = url.split("/")
        if len(name_parts) != 2:
            return None, None
        workbook = name_parts[0]
        view = "{}/sheets/{}".format(workbook, name_parts[1])
        return view, workbook

    @staticmethod
    def save_to_file(logger, output, filename):
        logger.info(_("httputils.found_attachment").format(filename))
        with open(filename, "wb") as f:
            f.write(output)
            logger.info(_("export.success").format(filename, ""))



import unittest

from src.commands.site.list_sites_command import ListSiteCommand
from .common_setup import *

commandname = "listsites"


class ListSitesParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, ListSiteCommand)

    def test_list_site_parser(self):
        mock_args = [commandname]
        args = self.parser_under_test.parse_args(mock_args)
        assert args is not None

    def test_list_site_parser_user_quota_integer(self):
        mock_args = [commandname, "--get-extract-encryption-mode"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.get_extract_encryption_mode is True



from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.execution.localize import _
from src.execution.logger_config import log
from .datasources_and_workbooks_command import DatasourcesAndWorkbooks


class RunSchedule(DatasourcesAndWorkbooks):
    """
    This command runs the specified schedule as it is on the server.
    """

    name: str = "runschedule"
    description: str = _("runschedule.short_description")

    @staticmethod
    def define_args(runschedule_parser):
        runschedule_parser.add_argument("schedule", help=_("tabcmd.run_schedule.options.schedule"))

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        logger.info(_("export.status").format(args.schedule))
        schedule = DatasourcesAndWorkbooks.get_items_by_name(logger, server.schedules, args.schedule)[0]
        if not schedule:
            Errors.exit_with_error(logger, _("publish.errors.server_resource_not_found"))
        logger.info(_("runschedule.status"))
        Errors.exit_with_error(logger, "Not yet implemented")

        # TODO implement in REST/tsc



import argparse
import unittest
from unittest import mock

from src.commands.auth.logout_command import LogoutCommand
from .common_setup import *

commandname = "logout"


class LogoutParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, LogoutCommand)

    # Logout doesn't take any arguments, so it's a good one to check all the global args on

    @mock.patch(
        "sys.argv",
        argparse.Namespace(
            server="https://localhost/",
            username="helloworld",
            site="",
            logging_level="info",
            password="testing123",
            no_prompt=True,
            token=None,
            token_name=None,
            cookie=True,
            no_cookie=False,
            prompt=False,
        ),
    )
    def test_logout_username_password(self):
        mock_args = [
            commandname,
            "--server",
            "the-server",
            "--site",
            "the-site",
            "--username",
            "me",
            "--no-certcheck",
            "-x",
            "proxyname",
        ]
        args = self.parser_under_test.parse_args(mock_args)

    def test_logout_missing_server(self):
        mock_args = [commandname, "--logging-level", "DEBUG", "--username", "me"]
        args = self.parser_under_test.parse_args(mock_args)
        assert not args.server, args
        assert args.logging_level == "DEBUG", args

    def test_logout_missing_username(self):
        mock_args = [commandname, "--server", "the-server"]
        args = self.parser_under_test.parse_args(mock_args)



import sys

try:
    from src.tabcmd import main
except ImportError:
    print("Tabcmd needs to be run as a module, it cannot be run as a script")
    print("Try running python -m tabcmd")
    sys.exit(1)

if __name__ == "__main__":
    main()



import unittest

from src.commands.group.create_group_command import CreateGroupCommand
from .common_setup import *

commandname = "creategroup"


class CreateGroupParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, CreateGroupCommand)

    def test_creategroup_parser_required_name(self):
        mock_args = [commandname, "name"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.name == "name"

    def test_creategroup_parser_missing_all_args(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            self.parser_under_test.parse_args(mock_args)



import unittest

from src.commands.project.publish_samples_command import PublishSamplesCommand
from .common_setup import *

commandname = "publishsamples"


class PublishParserParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, PublishSamplesCommand)

    def test_publish_samples_parser_name(self):
        mock_args = [commandname, "-n", "project"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.project_name == "project", args

    def test_publish_samples_parser_missing_all_args(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_publish_samples_parser_optional_args(self):
        mock_args = [
            commandname,
            "--name",
            "project",
            "--parent-project-path",
            "parent",
        ]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.parent_project_path == "parent", args
        assert args.project_name == "project", args



import gettext
import unittest
from src.execution.localize import set_client_locale


class LocaleTests(unittest.TestCase):
    def test_defaults(self):
        translations = set_client_locale()
        assert translations is not None
        # False: bound method != function ?
        # assert translations == gettext.NullTranslations.gettext

    def test_en_smoke_launching(self):
        # 27 incidents of this string
        translations = set_client_locale()
        assert translations is not None

        assert translations("tabcmd.launching") == "======================= Launching command ======================="

    def test_en_smoke_publish_errors(self):
        # 8 incidents of this string
        translations = set_client_locale()
        assert translations is not None

        assert translations("publish.errors.unexpected_server_response") == "Unexpected response from the server: {0}"

    def test_en_smoke_output_succeeded(self):
        # 8 incidents of this string
        translations = set_client_locale()
        assert translations is not None

        assert translations("common.output.succeeded") == "Succeeded"

    def test_en_smoke_percent_complete(self):
        # 5 incidents of this string
        translations = set_client_locale()
        assert translations is not None

        assert translations("session.monitorjob.percent_complete") == "{0}% complete"

    def test_en_smoke_line_processed(self):
        # 5 incidents of this string
        translations = set_client_locale()
        assert translations is not None

        assert translations("importcsvsummary.line.processed") == "Lines processed: {0}"



import sys

from src.execution.tabcmd_controller import TabcmdController


def main():

    if sys.version_info < (3, 7):
        raise ImportError(
            "Tabcmd requires Python 3.7 but you are on " + sys.version_info + " - please update your python version."
        )

    try:
        parser = TabcmdController.initialize()
        TabcmdController.run(parser)
    except Exception as e:
        print("Unhandled exception: {}".format(type(e).__name__))
        print(
            f"{type(e).__name__} at line {e.__traceback__.tb_lineno} of {__file__}: {e}",
        )
        sys.exit(1)


if __name__ == "__main__":
    main()



from argparse import Namespace
import unittest
from unittest import mock
from unittest.mock import patch, mock_open

from src.commands.auth.session import Session
import os

args_to_mock = Namespace(
    username=None,
    site_name=None,
    password=None,
    password_file=None,
    server=None,
    token_name=None,
    token=None,
    logging_level=None,
    no_certcheck=None,
    no_prompt=False,
    no_cookie=False,
    certificate=None,
    proxy=None,
    no_proxy=True,
    timeout=None,
)

mock_data_from_json = Namespace(
    token=None,  # this is the session token, not pat
    username=None,
    user_id=None,
    site_name=None,
    site_id=None,
    password=None,
    password_file=None,
    server=None,
    personal_access_token_name=None,
    personal_access_token=None,
    last_login_using=None,
    no_cookie=False,
    certificate=None,
    no_cert=True,
    proxy=None,
    no_proxy=True,
    timeout=None,
    no_prompt=False,
)

fakeserver = "http://SRVR"


def _set_mocks_for_json_file_saved_username(mock_json_load, auth_token, username):
    mock_auth = vars(mock_data_from_json)
    mock_json_load.return_value = {"tableau_auth": [mock_auth]}
    mock_auth["auth_token"] = auth_token
    mock_auth["username"] = username
    mock_auth["server"] = fakeserver
    mock_auth["last_login_using"] = "username"
    mock_auth["no_certcheck"] = True


def _set_mocks_for_json_file_exists(mock_path, does_it_exist=True):
    os.path = mock_path
    mock_path.expanduser.return_value = ""
    mock_path.join.return_value = ""
    mock_path.exists.return_value = does_it_exist
    return mock_path


def _set_mocks_for_creds_file(mock_file):
    mock_file.readlines.return_value = "dummypassword"
    return mock_file


@mock.patch("json.dump")
@mock.patch("json.load")
@mock.patch("os.path")
@mock.patch("builtins.open")
class JsonTests(unittest.TestCase):
    def test_read_session_from_json(self, mock_open, mock_path, mock_load, mock_dump):
        _set_mocks_for_json_file_exists(mock_path)
        _set_mocks_for_json_file_saved_username(mock_load, "AUTHTOKEN", "USERNAME")
        test_session = Session()
        test_session._read_from_json()
        assert hasattr(test_session.auth_token, "AUTHTOKEN") is False, test_session
        assert hasattr(test_session, "password") is False, test_session
        assert test_session.username == "USERNAME"
        assert test_session.server_url == fakeserver, test_session.server_url

    def test_save_session_to_json(self, mock_open, mock_path, mock_load, mock_dump):
        _set_mocks_for_json_file_exists(mock_path)
        test_session = Session()
        test_session.username = "USN"
        test_session.server = "SRVR"
        test_session._save_session_to_json()
        assert mock_dump.was_called()


@mock.patch("getpass.getpass")
class BuildCredentialsTests(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # make sure any json data etc is cleared
        Session().end_session_and_clear_data()

    # Set: Create auth credentials
    # This first set of tests has no attributes set
    def test__create_new_token_credential_fails_no_args(self, mock_pass):
        active_session = Session()
        with self.assertRaises(SystemExit):
            active_session._create_new_token_credential()

    def test__create_new_username_credential_fails_no_args(self, mock_pass):
        active_session = Session()
        with self.assertRaises(SystemExit):
            active_session._create_new_credential(None, None)

    # These two already have a username saved and pass in a password as argument
    def test__create_new_token_credential_succeeds_new_token(self, mock_pass):
        test_args = Namespace(**vars(args_to_mock))
        test_args.token = "gibberish"
        active_session = Session()
        assert active_session.token is None, active_session.token
        active_session.token_name = "readable"
        auth = active_session._create_new_token_credential()
        assert auth is not None
        assert mock_pass.is_not_called()

    def test__create_new_username_credential_succeeds_new_password(self, mock_pass):
        test_password = "pword1"
        active_session = Session()
        active_session.username = "user"
        active_session.site = ""
        auth = active_session._create_new_credential(test_password, Session.PASSWORD_CRED_TYPE)
        assert auth is not None

    # this one has a token saved
    def test__create_new_token_credential_succeeds_from_self(self, mock_pass):
        active_session = Session()
        active_session.token = "gibberish2"
        active_session.token_name = "readable2"
        auth = active_session._create_new_token_credential()
        assert mock_pass.is_not_called()
        assert auth is not None
        assert auth.token_name == "readable2", auth
        assert auth.personal_access_token == "gibberish2", auth

    # this one calls getpass because we don't store the password
    def test__create_new_username_credential_succeeds_from_self(self, mock_pass):
        active_session = Session()
        active_session.username = "user3"
        active_session.site = ""
        auth = active_session._create_new_credential(None, Session.PASSWORD_CRED_TYPE)
        assert mock_pass.has_been_called()
        assert auth is not None
        assert auth.username == "user3", auth
        assert auth.password == mock_pass(), auth

    def test__create_new_token_credential_succeeds_from_args(self, mock_pass):
        test_args = Namespace(**vars(args_to_mock))
        test_args.token = "gibberish"
        test_args.token_name = "readable"
        active_session = Session()
        active_session._update_session_data(test_args)
        auth = active_session._create_new_token_credential()

    def test__create_new_username_credential_succeeds_from_args(self, mock_pass):
        test_args = Namespace(**vars(args_to_mock))
        test_args.username = "user"
        test_args.password = "pwordddddd"
        active_session = Session()
        active_session._update_session_data(test_args)
        auth = active_session._create_new_credential(test_args.password, Session.PASSWORD_CRED_TYPE)


class PromptingTests(unittest.TestCase):
    def test_show_prompt_if_user_didnt_say(self):
        test_args = Namespace(**vars(args_to_mock))
        assert Session._allow_prompt(test_args) is True, test_args

    def test_show_prompt_if_user_said_yes(self):
        test_args = Namespace(**vars(args_to_mock))
        test_args.prompt = True
        assert Session._allow_prompt(test_args) is True, test_args

    def test_dont_show_prompt_if_user_said_no(self):
        test_args = Namespace(**vars(args_to_mock))
        test_args.no_prompt = True
        assert Session._allow_prompt(test_args) is False, test_args


"""
These tests all call the top-level method create_session, and mock out
the json file, and the TSC.Server calls
"""


@mock.patch("json.dump")
@mock.patch("json.load")
@mock.patch("os.path")
@mock.patch("builtins.open")
@mock.patch("getpass.getpass")
class CreateSessionTests(unittest.TestCase):
    @mock.patch("tableauserverclient.Server")
    def test_create_session_first_time_no_args(
        self, mock_tsc, mock_pass, mock_file, mock_path, mock_json_load, mock_json_dump
    ):
        mock_path = _set_mocks_for_json_file_exists(mock_path, False)
        assert mock_path.exists("anything") is False
        test_args = Namespace(**vars(args_to_mock))
        mock_tsc().users.get_by_id.return_value = None
        new_session = Session()
        with self.assertRaises(SystemExit):
            auth = new_session.create_session(test_args)

    @mock.patch("tableauserverclient.Server")
    def test_create_session_first_time_with_token_arg(
        self, mock_tsc, mock_pass, mock_file, mock_path, mock_json_load, mock_json_dump
    ):
        mock_path = _set_mocks_for_json_file_exists(mock_path, False)
        assert mock_path.exists("anything") is False
        test_args = Namespace(**vars(args_to_mock))
        test_args.token_name = "tn"
        test_args.token = "foo"
        new_session = Session()
        auth = new_session.create_session(test_args)
        assert auth is not None, auth
        assert auth.auth_token is not None, auth.auth_token
        assert auth.auth_token.name is not None, auth.auth_token
        assert new_session.token == "foo", new_session.token
        assert new_session.token_name == "tn", new_session

    @mock.patch("tableauserverclient.Server")
    def test_create_session_first_time_with_password_arg(
        self, mock_tsc, mock_pass, mock_file, mock_path, mock_json_load, mock_json_dump
    ):
        mock_path = _set_mocks_for_json_file_exists(mock_path, False)
        test_args = Namespace(**vars(args_to_mock))
        test_args.username = "uuuu"
        test_args.password = "pppp"
        new_session = Session()

        auth = new_session.create_session(test_args)
        assert auth is not None, auth
        assert auth.auth_token is not None, auth.auth_token
        assert new_session.username == "uuuu", new_session
        assert mock_tsc.has_been_called()

    @mock.patch("tableauserverclient.Server")
    def test_create_session_first_time_with_password_file_as_password(
        self, mock_tsc, mock_pass, mock_file, mock_path, mock_json_load, mock_json_dump
    ):
        mock_path = _set_mocks_for_json_file_exists(mock_path, False)
        test_args = Namespace(**vars(args_to_mock))
        test_args.username = "uuuu"
        # filename = os.path.join(os.path.dirname(__file__),"test_credential_file.txt")
        # test_args.password_file = os.getcwd()+"/test_credential_file.txt"
        test_args.password_file = "filename"
        with mock.patch("builtins.open", mock.mock_open(read_data="my_password")):
            new_session = Session()
            auth = new_session.create_session(test_args)

        assert auth is not None, auth
        assert auth.auth_token is not None, auth.auth_token
        assert new_session.username == "uuuu", new_session
        assert new_session.password_file == "filename", new_session
        assert mock_tsc.has_been_called()

    @mock.patch("tableauserverclient.Server")
    def test_create_session_first_time_with_password_file_as_token(
        self, mock_tsc, mock_pass, mock_file, mock_path, mock_json_load, mock_json_dump
    ):
        mock_path = _set_mocks_for_json_file_exists(mock_path, False)
        test_args = Namespace(**vars(args_to_mock))
        test_args.token_name = "mytoken"
        test_args.password_file = "filename"
        with mock.patch("builtins.open", mock.mock_open(read_data="my_token")):
            new_session = Session()
            auth = new_session.create_session(test_args)

        assert auth is not None, auth
        assert auth.auth_token is not None, auth.auth_token
        assert new_session.password_file == "filename", new_session
        assert mock_tsc.has_been_called()

    @mock.patch("tableauserverclient.Server")
    def test_load_saved_session_data(self, mock_tsc, mock_pass, mock_file, mock_path, mock_json_load, mock_json_dump):
        _set_mocks_for_json_file_exists(mock_path, True)
        _set_mocks_for_json_file_saved_username(mock_json_load, "auth_token", "username")
        test_args = Namespace(**vars(args_to_mock))
        new_session = Session()
        new_session._read_existing_state()
        new_session._update_session_data(test_args)
        assert new_session, new_session
        assert new_session.username == "username", new_session.username
        assert new_session.server_url == fakeserver, new_session.server_url
        assert mock_tsc.has_been_called()

    @mock.patch("tableauserverclient.Server")
    def test_create_session_with_active_session_saved(
        self, mock_tsc, mock_pass, mock_file, mock_path, mock_json_load, mock_json_dump
    ):
        _set_mocks_for_json_file_exists(mock_path, True)
        _set_mocks_for_json_file_saved_username(mock_json_load, "auth_token", None)
        test_args = Namespace(**vars(args_to_mock))
        test_args.token = "tn"
        test_args.token_name = "tnnnn"
        test_args.no_prompt = False
        new_session = Session()

        auth = new_session.create_session(test_args)
        assert auth is not None, auth
        assert auth.auth_token is not None, auth.auth_token
        assert mock_tsc.has_been_called()

    @mock.patch("tableauserverclient.Server")
    def test_create_session_with_saved_expired_username_session(
        self, mock_tsc, mock_pass, mock_file, mock_path, mock_json_load, mock_json_dump
    ):
        _set_mocks_for_json_file_saved_username(mock_json_load, "auth_token", "username")
        _set_mocks_for_json_file_exists(mock_path, True)
        tsc_under_test = CreateSessionTests._set_mock_tsc_not_signed_in(mock_tsc)
        CreateSessionTests._set_mock_tsc_sign_in_succeeds(tsc_under_test)
        test_args = Namespace(**vars(args_to_mock))
        mock_pass.getpass.return_value = "success"
        test_args.password = "eqweqwe"
        new_session = Session()
        auth = new_session.create_session(test_args)
        assert mock_pass.has_been_called()
        assert auth is not None, auth
        assert auth.auth_token is not None, auth.auth_token
        assert auth.auth_token == "cookiieeeee"
        assert new_session.username == "username", new_session
        assert mock_tsc.use_server_version.has_been_called()

    @staticmethod
    def _set_mock_tsc_not_signed_in(mock_tsc):
        tsc_in_test = mock.MagicMock(name="manually mocking tsc")
        mock_tsc.return_value = tsc_in_test
        tsc_in_test.is_signed_in.return_value = False  # CreateSessionTests.return_False
        tsc_in_test.server_info.get.return_value = Exception
        return tsc_in_test

    @staticmethod
    def _set_mock_tsc_sign_in_succeeds(mock_tsc):
        tscauth_mock = mock.MagicMock(name="tsc.auth")
        mock_tsc.auth = tscauth_mock
        mock_tsc.auth_token = "cookiieeeee"
        mock_tsc.site_id = "1"
        mock_tsc.user_id = "0"


@mock.patch("tableauserverclient.Server")
class ConnectionOptionsTest(unittest.TestCase):
    def test_certcheck_on(self, mock_tsc):
        mock_tsc.add_http_options = mock.MagicMock()
        mock_session = mock.MagicMock()
        mock_session.no_certcheck = True
        mock_session.site_id = "s"
        mock_session.user_id = "u"
        server = "anything"
        mock_session._set_connection_options()
        assert mock_tsc.add_http_options.has_been_called()

    def test_certcheck_off(self, mock_tsc):
        mock_session = mock.MagicMock()
        server = "anything"
        mock_session.site_id = "s"
        mock_session.user_id = "u"
        mock_session._set_connection_options()
        mock_tsc.add_http_options.assert_not_called()

    """
    def test_cert(self, mock_tsc):
        assert False, 'feature not implemented'

    def test_proxy_stuff(self, mock_tsc):
        assert False, 'feature not implemented'

    def test_timeout(self, mock_tsc):
        assert False, 'feature not implemented'

class CookieTests(unittest.TestCase):

    def test_no_file_if_no_cookie(self):
        assert False, 'feature not implemented'


class LastLoggedInTests(unittest.TestCase):

    def test_last_logged_in_username(self):
        assert False, 'test not implemented'

    def test_last_logged_in_token(self):
        assert False, 'test not implemented'

    def test_no_previous_login(self):
        assert False, 'test not implemented'
"""



import subprocess
import os
import pytest

try:
    from tests.e2e import credentials
except ImportError:
    credentials = None  # type: ignore

our_program = "tabcmd.exe"
launch_path = os.path.join("dist", "tabcmd")
exe = os.path.join(launch_path, our_program)


def login(extra="--language", value="en"):
    if not credentials:
        return
    # --server, --site, --username, --password
    args = [
        "python",
        "-m",
        "tabcmd",
        "login",
        "--server",
        credentials.server,
        "--site",
        credentials.site,
        "--token",
        credentials.token,
        "--token-name",
        credentials.token_name,
        "--no-certcheck",
        extra,
        value,
    ]
    print(args)
    return subprocess.check_call(args, stderr=subprocess.STDOUT, shell=True)


def get_executable():
    print("script is running in:")
    subprocess.check_call(["chdir"], shell=True)
    print("expecting built executable to be in " + launch_path + ":")
    subprocess.check_call(["dir", launch_path], shell=True)
    print("running", our_program)
    return exe



import glob
import os
import subprocess
import ftfy

LOCALES = [
    "en", "de", "es", "fr", "ga", "it", "pt",
    "sv", "ja", "ko",
    "zh"]

"""
https://pydoit.org/
Usage:
pip install doit
doit list # see available tasks

FYI: to read mo and po files use https://poedit.net/download
"""


def task_convert():
    """
    For all languages: Read properties files with unicode like "Schlie\u00dfen", save it back as "Schließen"
    requires: pip install ftfy
    help: https://ftfy.readthedocs.io/
    Run when we copy in updated properties files. NOT IDEMPOTENT: only run on the initial files
    """

    def process_locales():
        for current_locale in LOCALES:
            if current_locale in ["en", "ga"]:  # greek is our pseudo-loc
                continue
            elif current_locale in ["ja", "ko", "zh"]:
                encoding = "utf-8"
            else:
                encoding = "cp1252"

            for file in glob.glob("src/locales/" + current_locale + "/*.properties"):
                basename = os.path.basename(file).split(".")[0]
                print("transcoding", basename)
                with open(file, encoding=encoding) as infile:
                    data = infile.read()
                # now that we have read in the data properly encoded, fix the \u00fc characters and save as utf-8
                with open(file, "w", encoding="utf-8") as outfile:
                    outfile.write(ftfy.fixes.decode_escapes(data))

    return {
        'actions': [process_locales],
        'verbosity': 2,
    }


def task_po():
    """
    For all languages: generate a .po file from each .properties file
    Run when we copy in updated properties files AFTER task_convert (so all files are utf-8)
    This is idempotent and can be re-run safely
    """

    """
    There are two versions of prop2po:
    - 1.0, available through pip install prop2po, from https://github.com/mivek/prop2po
    it doesn't have any way to control which encoding it uses so I'm patching it
    - 3.x, from pip install translate-toolkit: 
    it copies key->comment, value-> msgid, ""->msgstr which is not at all what we want
    """
    def process_locales():
        for current_locale in LOCALES:

            LOC_PATH = "src/locales/" + current_locale
            for file in glob.glob(LOC_PATH+"/*.properties"):
                basename = os.path.basename(file).split(".")[0]
                print("processing", basename)
                result = subprocess.run(["python",
                                         "bin/i18n/prop2po.py",
                                         "--encoding", "utf-8",  # for the .po header
                                         "--language", current_locale,  # for the .po header
                                         LOC_PATH + "/"+basename+".properties",
                                         LOC_PATH + "/LC_MESSAGES/"+basename+".po"])
                print("\n", result)
                # print("stdout:", result.stdout)
                if not result.returncode == 0:
                    print("stderr:", result.stderr)
    return {
        'actions': [process_locales],
        'verbosity': 2,
    }


def task_clean_all():
    """For all languages: removes all generated artifacts (.po, .mo) which source from properties files. """

    def process_locales():
        for current_locale in LOCALES:
            LOC_PATH = "src/locales/" + current_locale
            for file in glob.glob(LOC_PATH+"/*.properties"):
                basename = os.path.basename(file).split(".")[0]
                print("deleting",basename + ".*")
                try:
                    os.remove(LOC_PATH + "/LC_MESSAGES/"+basename+".po")
                except OSError:
                    pass
                try:
                    os.remove(LOC_PATH + "/LC_MESSAGES/" + basename + ".mo")
                except OSError:
                    pass
            try:
                print("deleting", current_locale + ".mo")
                os.remove(LOC_PATH + "/LC_MESSAGES/" + current_locale + ".mo")
            except OSError:
                pass


    return {
        'actions': [process_locales],
        'verbosity': 2,
    }


def task_merge():
    """
    For all languages: Combines all existing po files for a language into a single domain called 'tabcmd'.
    """

    def process_locales():
        for current_locale in LOCALES:

            LOC_PATH = "src/locales/" + current_locale + "/LC_MESSAGES"

            with open(LOC_PATH + "/tabcmd.po", 'w+', encoding="utf-8") as outfile:
                for file in glob.glob(LOC_PATH + "/*.po"):
                    if file.endswith("tabcmd.po"):
                        pass
                    else:
                        print("merging", file)
                        with open(file, encoding="utf-8") as infile:
                            outfile.write(infile.read())
                            outfile.write("\n")

    return {
        'actions': [process_locales],
        'verbosity': 2,
    }


def task_mo():
    """
    For all languages: Processes the tabcmd.po file to produce a final tabcmd.mo file for each language
    Uses msgfmt.py from gettext, which is copied locally into the repo
    """

    def process_locales():
        for current_locale in LOCALES:

            LOC_PATH = "src/locales/" + current_locale + "/LC_MESSAGES"

            print("writing final tabcmd.mo file")
            # build the single binary file from the .po file
            result = subprocess.run(["python", "bin/i18n/msgfmt.py", LOC_PATH + "/src"])
            print("\n", result)
            # print("stdout:", result.stdout)
            if not result.returncode == 0:
                print("stderr:", result.stderr)

    return {
        'actions': [process_locales],
        'verbosity': 2,
    }








import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log


class DeleteExtracts(Server):
    """
    Command to delete extracts for a published workbook or data source.
    """

    name: str = "deleteextracts"
    description: str = _("deleteextracts.short_description")

    @staticmethod
    def define_args(delete_extract_parser):
        set_ds_xor_wb_args(delete_extract_parser)
        set_embedded_datasources_options(delete_extract_parser)
        # set_encryption_option(delete_extract_parser)
        set_project_arg(delete_extract_parser)
        set_parent_project_arg(delete_extract_parser)
        delete_extract_parser.add_argument("--url", help=_("createextracts.options.url"))

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        try:
            if args.datasource:
                logger.info(_("deleteextracts.for.datasource").format(args.datasource))
                data_source_item = Server.get_data_source_item(logger, server, args.datasource)
                job = server.datasources.delete_extract(data_source_item)
            elif args.workbook:
                logger.info(_("deleteextracts.for.workbook_name").format(args.workbook))
                workbook_item = Server.get_workbook_item(logger, server, args.workbook)
                job = server.workbooks.delete_extract(workbook_item)
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, _("deleteextracts.errors.error"), e)

        logger.info(_("common.output.job_queued_success"))
        logger.debug("Extract deletion queued with JobID: {}".format(job.id))



import unittest

from src.commands.site.delete_site_command import DeleteSiteCommand
from .common_setup import *

commandname = "deletesite"


class DeleteSiteParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, DeleteSiteCommand)

    def test_delete_site(self):
        mock_args = [commandname, "site-name"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.site_name == "site-name", args

    def test_delete_site_required_name_none(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)



import argparse

from .localize import _

"""
- all arguments in here are optional, must be --argument
- all methods in here should return a parser for nice fluent chaining

Method naming conventions
- I have named methods in here aaaa_arg if it returns a value we want, aaa_option if it's a flag
- if not otherwise specified, the value passed in with the arg will be saved as args.argname
- options that just need to turn on or off a flag should be set with action=store_True/False
- for optional arguments, try to use argname: --site-name gets saved as variablename: site_name

All these optional arguments could be used with any command. 
Ideally *all* optional args would be in here, so they can be kept updated together and simply listed on the relevant parser.

Positional arguments will generally be set directly in each parser
Naming note: cannot have hyphens in positional args 
SO define them as add_argument([name for args attribute], metavar=[name to show in help])
e.g add_argument(site, metavar='site-name')
FOR site-name, project-name, workbook-name, datasource-name, group-name, schedule-name, token-name
-> in args: site, project, workbook, ....
BUT filename, username -> filename, username in command/parser

"""


def set_parent_project_arg(parser):
    parser.add_argument("--parent-project-path", default=None, help="path of parent project")
    return parser


# add/remove-user
# parser.users command has a bunch of deprecated options. Just ditch them?
# --admin-type, --[no-]publisher, --[no-]complete (which was ssuuuuubtly different from add-users)


def set_users_file_arg(parser):
    parser.add_argument(
        "--users",
        required=True,
        type=argparse.FileType("r", encoding="UTF-8"),
        help="CSV file containing a list of users.",
    )
    return parser


def set_users_file_positional(parser):
    parser.add_argument(
        "filename",
        metavar="filename.csv",
        type=argparse.FileType("r", encoding="UTF-8"),
        help="CSV file containing a list of users.",
    )
    return parser


def set_no_wait_option(parser):
    parser.add_argument(
        "--no-wait",
        action="store_true",
        help="Do not wait for asynchronous jobs to complete.",
    )
    return parser


def set_role_arg(parser):
    parser.add_argument(
        "-r",
        "--role",
        choices=[
            "ServerAdministrator",
            "SiteAdministratorCreator",
            "SiteAdministratorExplorer",
            "SiteAdministrator",
            "Creator",
            "ExplorerCanPublish",
            "Publisher",
            "Explorer",
            "Interactor",
            "Viewer",
            "Unlicensed",
        ],
        help="Specifies a site role for all users in the .csv file.",
    )
    return parser


def set_silent_option(parser):
    parser.add_argument("--silent-progress", help="Do not display progress messages for the command.")
    return parser


def set_completeness_options(parser):
    completeness_group = parser.add_mutually_exclusive_group()
    completeness_group.add_argument(
        "--complete",
        dest="require_all_valid",
        action="store_true",
        help="Requires that all rows be valid for any change to succeed.",
    )
    completeness_group.add_argument(
        "--no-complete",
        dest="require_all_valid",
        action="store_false",
        help="Allows a change to succeed when not all rows are valid. If not specified --complete is used.",
    )
    completeness_group.set_defaults(require_all_valid=True)
    return parser


# used in create/delete extract
# docs don't say it, but --embedded-datasources and --include-all could be mutually exclusive
def set_embedded_datasources_options(parser):
    embedded_group = parser.add_mutually_exclusive_group()
    embedded_group.add_argument(
        "--embedded-datasources",
        help="A space-separated list of embedded data source names within the target workbook.",
    )
    embedded_group.add_argument(
        "--include-all",
        action="store_true",
        help="Include all embedded data sources within target workbook.",
    )
    return parser


# used in create extract. listed in delete-extract but makes no sense there
def set_encryption_option(parser):
    parser.add_argument(
        "--encrypt",
        dest="encrypt",
        action="store_false",
        help="Encrypt the newly created extract.",
    )
    return parser


# item arguments: datasource, workbook, project, url ...

# for some reason in parser.project, publish-samples it uses -n for destination project name
# for publish it uses -r for destination project name
# but parser.site uses -r for site-content-url
def set_project_r_arg(parser):
    parser.add_argument(
        "--project",
        "-r",
        dest="project_name",
        default="",
        help="The name of the project.",
    )
    return parser


def set_project_n_arg(parser):
    parser.add_argument(
        "-n",
        "--project",
        dest="project_name",
        default="",
        help="The name of the project.",
    )
    return parser


def set_project_arg(parser):
    parser.add_argument("--project", dest="project_name", default="", help="The name of the project.")
    return parser


def set_datasource_arg(parser, action="store_true"):
    parser.add_argument("-d", "--datasource", help="The name of the target data source.", action=action)
    return parser


def set_site_url_arg(parser):
    parser.add_argument("--url", help="The canonical name for the resource as it appears in the URL")
    return parser


def set_workbook_arg(parser, action="store_true"):  # true if the user adds --workbook
    parser.add_argument("-w", "--workbook", help="The name of the target workbook.", action=action)
    return parser


def set_ds_xor_wb_args(parser):
    target_type_group = parser.add_mutually_exclusive_group(required=True)
    target_type_group.add_argument("-d", "--datasource", help="The name of the target datasource.")
    target_type_group.add_argument("-w", "--workbook", help="The name of the target workbook.")
    return parser


def set_description_arg(parser):
    parser.add_argument("-d", "--description", help="Specifies a description for the item.")
    return parser


# only in edit-site
def set_site_status_arg(parser):
    parser.add_argument(
        "--status",
        choices=["ACTIVE", "SUSPENDED"],
        help="Set to ACTIVE to activate a site, or to SUSPENDED to suspend a site.",
    )
    return parser


# create-site/update-site - lots of these options are never used elsewhere
# mismatched arguments: createsite says --url, editsite says --site-id
# just let both commands use either of them
def set_site_id_options(parser):
    site_id = parser.add_mutually_exclusive_group()
    site_id.add_argument("--site-id", help="Used in the URL to uniquely identify the site.")
    site_id.add_argument(
        "-r",
        "--url",
        help="Used in URLs to specify the site. Different from the site name.",
    )
    return parser


# these options are all shared in create-site and edit-site
def set_common_site_args(parser):

    parser = set_site_id_options(parser)

    parser.add_argument("--user-quota", type=int, help="Maximum number of users that can be added to the site.")

    site_help = "Allows or denies site administrators the ability to add users to or remove users from the site."
    site_group = parser.add_mutually_exclusive_group()
    site_group.add_argument(
        "--site-mode",
        dest="site_admin_user_management",
        action="store_true",
        help=site_help,
    )
    site_group.add_argument(
        "--no-site-mode",
        dest="site_admin_user_management",
        action="store_false",
        help=site_help,
    )

    parser.add_argument(
        "--storage-quota",
        type=int,
        help="In MB, the amount of data that can be stored on the site.",
    )

    parser.add_argument(
        "--extract-encryption-mode",
        choices=["enforced", "enabled", "disabled"],
        help="The extract encryption mode for the site can be enforced, enabled or disabled. ",
    )

    parser.add_argument(
        "--run-now-enabled",
        help="Allow or deny users from running extract refreshes, flows, or schedules manually. \
            true to allow users to run tasks manually or false to prevent users from running tasks manually.",
    )
    return parser


# this option is only used by listsites
def set_view_site_encryption(parser):
    parser.add_argument(
        "--get-extract-encryption-mode",
        action="store_true",
        help="Include the extract encryption mode for each site.",
    )


# export --- mmmaaaannnyyyy options
def set_filename_arg(parser, description=_("get.options.file")):
    parser.add_argument("-f", "--filename", help=description)


def set_publish_args(parser):
    parser.add_argument("-n", "--name", help="Name to publish the new datasource or workbook by.")

    append_group = parser.add_mutually_exclusive_group()
    append_group.add_argument(
        "-o",
        "--overwrite",
        action="store_true",
        help="Overwrites the workbook, data source, or data extract if it already exists on the server.",
    )
    append_group.add_argument(
        "--append",
        action="store_true",
        help="Append the extract file to the existing data source.",
    )
    parser.add_argument(
        "--db-username",
        help="Use this option to publish a database user name with the workbook, data source, or data extract.",
    )
    parser.add_argument(
        "--db-password",
        help="publish a database password with the workbook, data source, or extract",
    )
    parser.add_argument(
        "--save-db-password",
        help="Stores the provided database password on the server.",
    )
    parser.add_argument(
        "--tabbed",
        action="store_true",
        help="When a workbook with tabbed views is published, each sheet becomes a tab that viewers can use to \
        navigate through the workbook",
    )
    parser.add_argument("--replace", help="Use the extract file to replace the existing data source.")
    parser.add_argument("--disable-uploader", help="Disable the incremental file uploader.")
    parser.add_argument("--restart", help="Restart the file upload.")
    parser.add_argument(
        "--encrypt-extracts",
        help="Encrypt extracts in the workbook, datasource, or extract being published to the server",
    )
    parser.add_argument("--oauth-username", help="The email address of a preconfigured OAuth connection")
    parser.add_argument("--save-oauth")
    parser.add_argument("--thumbnail-username")
    parser.add_argument("--thumbnail-group")  # not implemented in the REST API


# refresh-extracts
def set_incremental_options(parser):
    sync_group = parser.add_mutually_exclusive_group()
    sync_group.add_argument("--incremental", help="Runs the incremental refresh operation.")
    sync_group.add_argument(
        "--synchronous",
        help="Adds the full refresh operation to the queue used by the Backgrounder process, to be run as soon as a \
        Backgrounder process is available.",
    )
    return parser


def set_calculations_options(parser):
    calc_group = parser.add_mutually_exclusive_group()
    calc_group.add_argument(
        "--addcalculations",
        action="store_true",
        help="DEPRECATED [has no effect] Add precalculated data operations in the extract data source.",
    )
    calc_group.add_argument(
        "--removecalculations",
        action="store_true",
        help="DEPRECATED [has no effect] Remove precalculated data in the extract data source.",
    )
    return calc_group


# TODO below
# these are not used in any Online operations, on the backburner


# edit-domain: none of these are used in other commands
def set_domain_arguments(parser):
    parser.add_argument(
        "--id",
        help="The ID of domain to change. To get a list of domain IDs, use use listdomains.",
    )
    parser.add_argument("--name", help="The new name for the domain.")
    parser.add_argument("--nickname", help="The new nickname for the domain.")
    return parser


# reset-openid-sub
def set_target_users_arg(parser):
    target_users_group = parser.add_mutually_exclusive_group()
    target_users_group.add_argument("--target-username", help="Clears sub value for the specified individual user.")
    target_users_group.add_argument("--all", help="Clears sub values for all users.")
    return parser


# set setting
# choices: allow_scheduling, embedded_credentials, remember_passwords_forever
# use !setting-name to disable them
# hmmmm


# sync-group
def set_update_group_args(parser):
    parser.add_argument(
        "--grant-license-mode",
        choices=["on-login", "on-sync"],
        help="Specifies whether a role should be granted on sign in. ",
    )
    parser.add_argument(
        "--overwritesiterole",
        action="store_true",
        help="Allows a user’s site role to be overwritten with a less privileged one when using --role.",
    )
    return parser


def set_upgrade_stop_option(parser):
    parser.add_argument(
        "--stop",
        action="store_true",
        help="When specified, stops the in progress Upgrade Thumbnails job.",
    )
    return parser


# validate-idp-metadata
# TODO not sure how these space-separated lists will work
def set_validate_idp_options(parser):
    parser.add_argument(
        "--digest-algorithms",
        metavar="<ALGORITHMS>",
        help="A space-separated list of digest algorithms. Legal values are sha1and sha256. \
            If not specified, server uses values from server configuration setting, \
            wgserver.saml.blocklisted_digest_algorithms.",
    )
    parser.add_argument(
        "--min-allowed-elliptic-curve-size",
        metavar="<SIZE>",
        help="If not specified, server uses values from server configuration setting, \
        wgserver.saml.min_allowed.elliptic_curve_size.",
    )
    parser.add_argument(
        "--min-allowed-rsa-key-size",
        metavar="<SIZE>",
        help="If not specified, server uses values from server configuration setting, \
        wgserver.saml.min_allowed.rsa_key_size.",
    )
    parser.add_argument(
        "--site-names",
        metavar="<SITENAMES>",
        help="A space-separated list of site names on which to perform certificate validation. \
        If not specified, then all sites are inspected.",
    )



import unittest

from src.commands.datasources_and_workbooks.publish_command import PublishCommand
from .common_setup import *

commandname = "Publish"


class PublishParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, PublishCommand)

    def test_publish_parser_required_name(self):
        mock_args = [commandname, "filename.hyper"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.filename == "filename.hyper", args

    def test_publish_parser_missing_all_args(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_publish_parser_tabbed(self):
        mock_args = [commandname, "filename.twbx", "--tabbed"]
        args = self.parser_under_test.parse_args(mock_args)
        assert args.tabbed is True, args



import unittest
from unittest import mock

from src.commands.user.delete_site_users_command import DeleteSiteUsersCommand
from .common_setup import *

commandname = "deletesiteusers"


class DeleteSiteUsersParserTest(unittest.TestCase):
    csv = ("testname", "testpassword", "test", "test", "test", "test")

    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, DeleteSiteUsersCommand)

    def test_delete_site_user_parser(self):
        with mock.patch("builtins.open", mock.mock_open(read_data="test")) as open_file:
            mock_args = [commandname, "users.csv"]
            args = self.parser_under_test.parse_args(mock_args)
            open_file.assert_called_with("users.csv", "r", -1, "UTF-8", None)

    def test_delete_site_user_parser_missing_arguments(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)



import argparse
import unittest
from unittest.mock import *
import tableauserverclient as TSC

from src.commands.auth import login_command, logout_command
from src.commands.datasources_and_workbooks import (
    delete_command,
    export_command,
    get_url_command,
    publish_command,
    runschedule_command,
)
from src.commands.extracts import (
    create_extracts_command,
    delete_extracts_command,
    decrypt_extracts_command,
    encrypt_extracts_command,
    reencrypt_extracts_command,
    refresh_extracts_command,
)
from src.commands.group import create_group_command, delete_group_command
from src.commands.help import help_command
from src.commands.project import create_project_command, delete_project_command, publish_samples_command
from src.commands.site import (
    create_site_command,
    delete_site_command,
    edit_site_command,
    list_sites_command,
)
from src.commands.user import (
    add_users_command,
    create_site_users,
    create_users_command,
    remove_users_command,
    delete_site_users_command,
)
from typing import List, NamedTuple, TextIO, Union
import io

mock_args = argparse.Namespace()
mock_args.logging_level = "info"

fake_item = MagicMock()
fake_item.name = "fake-name"
fake_item.id = "fake-id"
fake_item.pdf = b"/pdf-representation-of-view"
fake_item.extract_encryption_mode = "Disabled"

fake_job = MagicMock()
fake_job.id = "fake-job-id"

creator = MagicMock()
getter = MagicMock()
getter.get = MagicMock("get", return_value=([fake_item], 1))
getter.publish = MagicMock("publish", return_value=fake_item)
getter.create_extract = MagicMock("create_extract", return_value=fake_job)
getter.decrypt_extract = MagicMock("decrypt_extract", return_value=fake_job)
getter.delete_extract = MagicMock("delete_extract", return_value=fake_job)
getter.encrypt_extracts = MagicMock("encrypt_extracts", return_value=fake_job)
getter.reencrypt_extract = MagicMock("reencrypt_extract", return_value=fake_job)
getter.refresh = MagicMock("refresh", return_value=fake_job)


@patch("tableauserverclient.Server")
@patch("src.commands.auth.session.Session.create_session")
class RunCommandsTest(unittest.TestCase):
    @staticmethod
    def _set_up_session(mock_session, mock_server):
        mock_session.return_value = mock_server
        assert mock_session is not None
        mock_session.assert_not_called()
        global mock_args
        mock_args = argparse.Namespace(logging_level="DEBUG")

    # auth
    def test_login(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        login_command.LoginCommand.run_command(mock_args)
        mock_session.assert_called_with(mock_args)

    @patch("src.commands.auth.session.Session.end_session_and_clear_data")
    def test_logout(self, mock_end_session, mock_create_session, mock_server):
        logout_command.LogoutCommand.run_command(mock_args)
        mock_create_session.assert_not_called()
        mock_end_session.assert_called()

    def test_delete(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.workbooks = getter
        mock_server.datasources = getter
        mock_args.workbook = True
        mock_args.datasource = False
        mock_args.name = "name for on server"
        delete_command.DeleteCommand.run_command(mock_args)

    def test_export(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.workbooks = getter
        mock_args.fullpdf = True
        mock_args.filename = "filename.pdf"
        mock_args.url = "workbook-name/view-name"
        export_command.ExportCommand.run_command(mock_args)
        mock_session.assert_called()

    def test_get(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.views = getter
        mock_args.url = "/views/workbook-name/view-name"
        mock_args.filename = "filename.pdf"
        get_url_command.GetUrl.run_command(mock_args)
        mock_session.assert_called()

    def test_get_workbook(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.workbooks = getter
        mock_args.url = "/workbooks/workbook-name"
        mock_args.filename = "filename.twbx"
        get_url_command.GetUrl.run_command(mock_args)
        mock_session.assert_called()

    def test_publish(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_args.overwrite = False
        mock_args.filename = "existing_file.twbx"
        mock_args.project_name = "project-name"
        mock_args.parent_project_path = "projects"
        mock_args.name = ""
        mock_args.tabbed = True
        mock_server.projects = getter
        publish_command.PublishCommand.run_command(mock_args)
        mock_session.assert_called()

    @unittest.skip("target code not implemented yet")
    def test_runschedule(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.schedules = getter
        mock_args.schedule = "myschedule"
        with self.assertRaises(SystemExit):
            runschedule_command.RunSchedule.run_command(mock_args)
        # mock_session.assert_called()

    # extracts
    def test_create_extract(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.workbooks = getter
        mock_args.encrypt = False
        mock_args.include_all = True
        mock_args.datasource = None
        mock_args.embedded_datasources = None
        mock_args.workbook = "workbook"
        create_extracts_command.CreateExtracts.run_command(mock_args)
        mock_session.assert_called()

    def test_decrypt(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_args.site_name = "mock site"
        mock_server.sites = getter
        decrypt_extracts_command.DecryptExtracts.run_command(mock_args)
        mock_session.assert_called()

    def test_delete_extract(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.datasources = getter
        mock_args.datasource = True
        delete_extracts_command.DeleteExtracts.run_command(mock_args)
        mock_session.assert_called()

    def test_encrypt_extract(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.sites = getter
        mock_args.site_name = "name"
        encrypt_extracts_command.EncryptExtracts.run_command(mock_args)
        mock_session.assert_called()

    def test_reencrypt_extract(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_args.site_name = "name"
        mock_server.sites = getter
        reencrypt_extracts_command.ReencryptExtracts.run_command(mock_args)
        mock_session.assert_called()

    def test_refresh_extract(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_args.datasource = "datasource"
        mock_server.datasources = getter
        mock_args.workbook = None
        mock_args.addcalculations = None
        mock_args.removecalculations = None
        mock_args.incremental = None
        mock_args.synchronous = None
        mock_args.project_name = None
        mock_args.parent_project_path = None
        refresh_extracts_command.RefreshExtracts.run_command(mock_args)
        mock_session.assert_called()

    # groups
    def test_create_group(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_args.name = "name"
        create_group_command.CreateGroupCommand.run_command(mock_args)
        mock_session.assert_called()

    # groups
    def test_create_group_already_exists(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_args.continue_if_exists = True

        mock_args.name = "name"
        mock_server.groups.create.return_value = TSC.ServerResponseError(409, "already exists", "detail")
        create_group_command.CreateGroupCommand.run_command(mock_args)
        mock_session.assert_called()

    def test_create_project_already_exists(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_args.continue_if_exists = True
        mock_args.project_name = "repeat"
        mock_args.parent_project_path = ""
        mock_args.description = "none"
        mock_server.projects.create.return_value = TSC.ServerResponseError(409, "already exists", "detail")
        create_project_command.CreateProjectCommand.run_command(mock_args)
        mock_session.assert_called()

    def test_create_site_already_exists(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_args.continue_if_exists = True
        mock_args.site_name = "duplicate"
        mock_args.url = "dplct"
        mock_args.admin_mode = None
        mock_args.user_quota = None
        mock_args.storage_quota = None
        mock_server.sites.create.return_value = TSC.ServerResponseError(409, "already exists", "detail")
        create_site_command.CreateSiteCommand.run_command(mock_args)
        mock_session.assert_called()

    def test_delete_group(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_args.name = "name"
        mock_server.groups = getter
        delete_group_command.DeleteGroupCommand.run_command(mock_args)
        mock_session.assert_called()

    # help
    def test_help(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        help_command.HelpCommand.run_command(mock_args)
        mock_session.assert_not_called()

    # project
    def test_create_project(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.projects = getter
        mock_args.project_name = "name"
        mock_args.description = ""
        mock_args.content_permission = None
        mock_args.parent_project_path = "projects"
        create_project_command.CreateProjectCommand.run_command(mock_args)
        mock_session.assert_called()

    def test_delete_project(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.projects = getter
        mock_args.project_name = "project-name"
        mock_session.assert_not_called()
        mock_args.parent_project_path = "projects"
        delete_project_command.DeleteProjectCommand.run_command(mock_args)
        mock_session.assert_called()

    def test_publish_samples(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.projects = getter
        mock_args.project_name = "gloop"
        mock_args.parent_project_path = ""
        publish_samples_command.PublishSamplesCommand.run_command(mock_args)
        mock_session.assert_called()

    # site
    def test_create_site(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_args.site_name = "site-name"
        mock_args.url = "site-content-url"
        mock_args.admin_mode = None
        mock_args.user_quota = (None,)
        mock_args.storage_quota = None
        create_site_command.CreateSiteCommand.run_command(mock_args)
        mock_session.assert_called()

    def test_delete_site(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.sites = getter
        mock_args.site_name = "site-name"
        delete_site_command.DeleteSiteCommand.run_command(mock_args)
        mock_session.assert_called()

    def test_edit_site(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.sites = getter
        mock_args.site_name = "site-name"
        mock_args.new_site_name = None
        mock_args.url = "new-url"
        mock_args.user_quota = "1"
        mock_args.storage_quota = "1"
        mock_args.status = "Suspended"
        mock_session.assert_not_called()
        edit_site_command.EditSiteCommand.run_command(mock_args)
        mock_session.assert_called()

    def test_list_sites(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.sites = getter
        mock_args.get_extract_encryption_mode = "Disabled"
        list_sites_command.ListSiteCommand.run_command(mock_args)
        mock_session.assert_called()

    # TODO: get typings for argparse
    class NamedObject(NamedTuple):
        name: str

    ArgparseFile = Union[TextIO, NamedObject]

    @staticmethod
    def _set_up_file(content=["Test", "", "Test", ""]) -> ArgparseFile:
        # the empty string represents EOF
        # the tests run through the file twice, first to validate then to fetch
        mock = MagicMock(io.TextIOWrapper)
        mock.readline.side_effect = content
        mock.name = "file-mock"
        return mock

    def test_add_users(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.groups = getter
        mock_server.users = getter
        mock_args.users = RunCommandsTest._set_up_file()
        mock_args.name = "the-group"
        mock_args.require_all_valid = False
        add_users_command.AddUserCommand.run_command(mock_args)
        mock_session.assert_called()

    def test_remove_users(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.groups = getter
        mock_server.users = getter
        mock_args.name = "group-removing-from"
        mock_args.users = RunCommandsTest._set_up_file()
        mock_args.require_all_valid = False
        remove_users_command.RemoveUserCommand.run_command(mock_args)
        mock_session.assert_called()

    def test_create_site_users(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_args.filename = RunCommandsTest._set_up_file()
        mock_args.require_all_valid = False
        mock_args.site_name = None
        create_site_users.CreateSiteUsersCommand.run_command(mock_args)
        mock_session.assert_called()

    def test_delete_site_users(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.users = getter
        mock_args.filename = RunCommandsTest._set_up_file()
        mock_args.site_name = None
        mock_args.require_all_valid = True
        delete_site_users_command.DeleteSiteUsersCommand.run_command(mock_args)
        mock_session.assert_called()

    def test_create_user(self, mock_session, mock_server):
        RunCommandsTest._set_up_session(mock_session, mock_server)
        mock_server.users = getter
        mock_args.filename = RunCommandsTest._set_up_file()
        mock_args.site_name = None
        mock_args.require_all_valid = False
        create_users_command.CreateUsersCommand.run_command(mock_args)
        mock_session.assert_called()



import argparse
import sys
import unittest
import mock
from src.execution.logger_config import *
from src.execution.tabcmd_controller import TabcmdController


class ExecutionTests(unittest.TestCase):
    @mock.patch("sys.argv", [""])
    def test_launch(self):
        parser = TabcmdController.initialize()
        TabcmdController.run(parser, ["help"])
        # check exit code = 0?

    def test_initialize(self):
        TabcmdController.initialize()

    @mock.patch("sys.argv", "")
    def test_inputs(self):
        fake_parser = mock.MagicMock(argparse.ArgumentParser)
        with self.assertRaises(SystemExit):
            TabcmdController.run(fake_parser, None)

    def test_invalid_command(self):
        parser = TabcmdController.initialize()
        # crashes out during parse_args
        with self.assertRaises(SystemExit):
            TabcmdController.run(parser, ["boo", "--language", "fr"])

    def test_launch_languages(self):
        parser = TabcmdController.initialize()
        # crashes out from list_sites.cmd
        with self.assertRaises(SystemExit):
            TabcmdController.run(parser, ["listsites", "--language", "fr"])

    def test_config_logger(self):
        configure_log("log_name", "DEBUG")
        configure_log("log_name", "INFO")
        configure_log("log_name", "ERROR")

    def test_get_logger(self):
        log("testing", "DEBUG")



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.localize import _
from src.execution.logger_config import log


class DecryptExtracts(Server):
    """Command that decrypts all extracts on a site. If no site is
    specified, extracts on the default site will be decrypted."""

    name: str = "decryptextracts"
    description: str = _("decryptextracts.short_description")

    @staticmethod
    def define_args(decrypt_extract_parser):
        decrypt_extract_parser.add_argument("site_name", metavar="site-name", help=_("editsite.options.site-name"))

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        site_item = Server.get_site_for_command_or_throw(logger, server, args)
        try:
            logger.info(_("decryptextracts.status").format(args.site_name))
            job = server.sites.decrypt_extracts(site_item.id)
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, "Error decrypting extracts", e)

        logger.info(_("common.output.job_queued_success"))
        logger.debug("Extract decryption queued with JobID: {}".format(job.id))



import os
import pytest
import subprocess
import time
import unittest

from tests.e2e import setup_e2e

debug_log = "--logging-level=DEBUG"
indexing_sleep_time = 1  # wait 1 second to confirm server has indexed updates

project_name = "not-default-name"
group_name = "test-ing-group"
workbook_name = "namebasic"

# to run this suite
# pytest -q tests/e2e/online_tests.py
# you can either run setup with a stored credentials file, or simply log in
# before running the suite so a session is active


def _test_command(test_args: list[str]):
    # this will raise an exception if it gets a non-zero return code
    # that should bubble up and fail the test?
    # dist/exe calling_args = [setup_e2e.exe] + test_args + [debug_log]
    calling_args = ["python", "-m", "tabcmd"] + test_args + [debug_log] + ["--no-certcheck"]
    print(calling_args)
    return subprocess.check_call(calling_args)


# This calls dist/tabcmd/tabcmd.exe
class OnlineCommandTest(unittest.TestCase):
    published = False
    gotten = False

    @classmethod
    def setup_class(cls):
        print("running python -m")
        # call this if we are using the built exe setup_e2e.prechecks()

    ## Individual methods that implement a command

    def _create_project(self, project_name, parent_path=None):
        command = "createproject"
        arguments = [command, "--name", project_name]
        if parent_path:
            arguments.append("--parent-project-path")
            arguments.append(parent_path)
        print(arguments)
        _test_command(arguments)

    def _delete_project(self, project_name, parent_path=None):
        command = "deleteproject"
        arguments = [command, project_name]
        if parent_path:
            arguments.append("--parent-project-path")
            arguments.append(parent_path)
        _test_command(arguments)

    def _publish_samples(self, project_name):
        command = "publishsamples"
        arguments = [command, "--name", project_name]
        _test_command(arguments)

    def _publish_wb(self, file, name):
        command = "publish"
        arguments = [command, file, "--name", name, "--overwrite"]
        return _test_command(arguments)

    def _delete_wb(self, file):
        command = "delete"
        arguments = [command, file]
        _test_command(arguments)

    def _get_view(self, wb_name_on_server, sheet_name):
        server_file = "/views/" + wb_name_on_server + "/" + sheet_name
        command = "get"
        arguments = [command, server_file]
        _test_command(arguments)

    def _get_custom_view(self):
        command = "get"

    def _get_view_with_filters(self):
        command = "get"

    def _get_workbook(self, server_file):
        command = "get"
        server_file = "/workbooks/" + server_file
        arguments = [command, server_file]
        _test_command(arguments)

    def _create_extract(self, wb_name):
        command = "createextracts"
        arguments = [command, "-w", wb_name, "--encrypt"]
        _test_command(arguments)

    # variation: url
    def _refresh_extract(self, wb_name):
        command = "refreshextracts"
        arguments = [command, "--workbook", wb_name]
        _test_command(arguments)

    def _delete_extract(self, wb_name):
        command = "deleteextracts"
        arguments = [command, "-w", wb_name]
        _test_command(arguments)

    # actual tests
    TWBX_FILE_WITH_EXTRACT = "extract-data-access.twbx"
    TWBX_WITH_EXTRACT_NAME = "WorkbookWithExtract"
    TWBX_WITH_EXTRACT_SHEET = "sheet1"
    TWBX_FILE_WITHOUT_EXTRACT = "simple-data.twbx"
    TWBX_WITHOUT_EXTRACT_NAME = "WorkbookWithoutExtract"
    TWBX_WITHOUT_EXTRACT_SHEET = "Testsheet1"

    @pytest.mark.order(1)
    def test_login(self):
        try:
            setup_e2e.login()
        except Exception as e:
            raise SystemExit(2)

    @pytest.mark.order(1)
    def test_version(self):
        command = ""
        arguments = [command]
        _test_command(arguments)

    @pytest.mark.order(1)
    def test_help(self):
        command = "help"
        arguments = [command]
        _test_command(arguments)

    @pytest.mark.order(2)
    def test_create_site_users(self):
        command = "createsiteusers"
        users = os.path.join("tests", "assets", "detailed_users.csv")
        arguments = [command, users, "--role", "Publisher"]
        _test_command(arguments)

    @pytest.mark.order(3)
    def test_creategroup(self):
        groupname = group_name
        command = "creategroup"
        arguments = [command, groupname]
        _test_command(arguments)

    @pytest.mark.order(4)
    def test_add_users_to_group(self):
        groupname = group_name
        command = "addusers"
        filename = os.path.join("tests", "assets", "usernames.csv")
        arguments = [command, groupname, "--users", filename]
        _test_command(arguments)

    @pytest.mark.order(5)
    def test_remove_users_to_group(self):
        groupname = group_name
        command = "removeusers"
        filename = os.path.join("tests", "assets", "usernames.csv")
        arguments = [command, groupname, "--users", filename]
        _test_command(arguments)

    @pytest.mark.order(6)
    def test_deletegroup(self):
        groupname = group_name
        command = "deletegroup"
        arguments = [command, groupname]
        _test_command(arguments)

    @pytest.mark.order(7)
    def test_publish_samples(self):
        project_name = "sample-proj"
        self._create_project(project_name)
        time.sleep(indexing_sleep_time)

        self._publish_samples(project_name)
        self._delete_project(project_name)

    @pytest.mark.order(8)
    def test_create_projects(self):
        # project 1
        self._create_project(project_name)
        time.sleep(indexing_sleep_time)
        # project 2
        self._create_project("project_name_2", project_name)
        time.sleep(indexing_sleep_time)
        # project 3
        parent_path = "{0}/{1}".format(project_name, project_name)
        self._create_project(project_name, parent_path)
        time.sleep(indexing_sleep_time)

    @pytest.mark.order(9)
    def test_delete_projects(self):
        self._delete_project("project_name_2", project_name)  # project 2
        self._delete_project(project_name)

    @pytest.mark.order(9)
    def test_publish_samples(self):
        self._publish_samples("Default")

    @pytest.mark.order(10)
    def test_publish(self):
        name_on_server = OnlineCommandTest.TWBX_WITH_EXTRACT_NAME
        file = os.path.join("tests", "assets", OnlineCommandTest.TWBX_FILE_WITH_EXTRACT)
        self._publish_wb(file, name_on_server)

    @pytest.mark.order(10)
    def test__get_wb(self):
        wb_name_on_server = OnlineCommandTest.TWBX_WITHOUT_EXTRACT_NAME
        self._get_workbook(wb_name_on_server + ".twbx")

    @pytest.mark.order(10)
    def test__get_wb(self):
        wb_name_on_server = OnlineCommandTest.TWBX_WITH_EXTRACT_NAME
        self._get_workbook(wb_name_on_server + ".twbx")

    @pytest.mark.order(10)
    def test__get_view(self):
        wb_name_on_server = OnlineCommandTest.TWBX_WITHOUT_EXTRACT_NAME
        sheet_name = OnlineCommandTest.TWBX_WITHOUT_EXTRACT_SHEET
        self._get_view(wb_name_on_server, sheet_name + ".pdf")

    @pytest.mark.order(10)
    def test__get_view_csv(self):
        wb_name_on_server = OnlineCommandTest.TWBX_WITH_EXTRACT_NAME
        sheet_name = OnlineCommandTest.TWBX_WITH_EXTRACT_SHEET
        self._get_view(wb_name_on_server, sheet_name + ".csv")

    @pytest.mark.order(10)
    def test__get_view_png(self):
        wb_name_on_server = OnlineCommandTest.TWBX_WITH_EXTRACT_NAME
        sheet_name = OnlineCommandTest.TWBX_WITH_EXTRACT_SHEET
        self._get_view(wb_name_on_server, sheet_name + ".png")

    @pytest.mark.order(11)
    def test__delete_wb(self):
        name_on_server = OnlineCommandTest.TWBX_WITH_EXTRACT_NAME
        self._delete_wb(name_on_server)

    @pytest.mark.order(12)
    def test_delete_extract(self):
        # fails because the extract has a bad data connection :/
        name_on_server = OnlineCommandTest.TWBX_WITH_EXTRACT_NAME
        file = os.path.join("tests", "assets", OnlineCommandTest.TWBX_FILE_WITH_EXTRACT)
        self._publish_wb(file, name_on_server)
        self._delete_extract(name_on_server)

    @pytest.mark.order(13)
    def test_create_extract(self):
        # Fails because it 'already has an extract' :/
        name_on_server = OnlineCommandTest.TWBX_WITHOUT_EXTRACT_NAME
        self._create_extract(name_on_server)

    @pytest.mark.order(14)
    def test_refresh_extract(self):
        name_on_server = OnlineCommandTest.TWBX_WITH_EXTRACT_NAME
        self._refresh_extract(name_on_server)
        self._delete_wb(name_on_server)

    def test_version(self):
        _test_command(["-v"])

    def test_help(self):
        _test_command(["help"])

    # this just gets in the way :(
    # def test_logout(self):
    #   _test_command(["logout"])

    @pytest.mark.order(15)
    def test_export_wb(self):
        name_on_server = OnlineCommandTest.TWBX_WITH_EXTRACT_NAME
        file = os.path.join("tests", "assets", OnlineCommandTest.TWBX_FILE_WITH_EXTRACT)
        self._publish_wb(file, name_on_server)
        command = "export"
        friendly_name = name_on_server + "/" + OnlineCommandTest.TWBX_WITH_EXTRACT_SHEET
        arguments = [command, friendly_name, "--fullpdf", "-f", "exported_wb.pdf"]
        _test_command(arguments)

    @pytest.mark.order(15)
    def test_export_view(self):
        name_on_server = OnlineCommandTest.TWBX_WITH_EXTRACT_NAME
        file = os.path.join("tests", "assets", OnlineCommandTest.TWBX_FILE_WITH_EXTRACT)
        self._publish_wb(file, name_on_server)
        command = "export"
        friendly_name = name_on_server + "/" + OnlineCommandTest.TWBX_WITH_EXTRACT_SHEET
        arguments = [command, friendly_name, "--pdf", "-f", "exported_view.pdf"]
        _test_command(arguments)

    @pytest.mark.order(16)
    def test_delete_site_users(self):
        command = "deletesiteusers"
        users = os.path.join("tests", "assets", "usernames.csv")
        _test_command([command, users])

    @pytest.mark.order(20)
    def test_list_sites(self):
        command = "listsites"
        _test_command([command])



import unittest
import argparse
from src.commands.datasources_and_workbooks.get_url_command import GetUrl
from .common_setup import *

commandname = "listsites"


class GetUrlParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, GetUrl)

    def test_get_url_parser_file(self):
        mock_args = vars(argparse.Namespace(filename="helloworld"))
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)

    def test_get_url_parser_missing_all_args(self):
        mock_args = [commandname]
        with self.assertRaises(SystemExit):
            args = self.parser_under_test.parse_args(mock_args)



import argparse
import logging
import pytest
import unittest
from src.commands.auth.session import Session
from src.commands.server import Server
from src.execution.logger_config import log


try:
    from tests.e2e import credentials
except ImportError:
    credentials = None  # type: ignore

fakeserver = "http://SRVR"
logging.disable(logging.ERROR)

# these are integration tests because they don't just run a command, they call interior methods
# pytest -v tests/e2e/integration_tests.py
class E2EJsonTests(unittest.TestCase):
    def test_save_then_read(self):
        test_session = Session()
        test_session.username = "USN"
        test_session.server_url = fakeserver
        test_session.password_file = "users.csv"
        test_session._save_session_to_json()
        new_session = Session()
        new_session._read_from_json()
        assert new_session.username == "USN", new_session.username
        assert hasattr(new_session, "password") is False, new_session
        assert new_session.server_url == fakeserver, new_session.server_url
        test_session.end_session_and_clear_data()


@pytest.mark.skipif(
    not credentials,
    reason="'No credentials file found to run tests against a live server",
)
class E2EServerTests(unittest.TestCase):

    saved_site_id = ""

    @staticmethod
    def test_log_in():
        if not credentials:
            return
        # TODO current test command doesn't recognize skips - change to proper pytest
        # TODO and then we can get rid of the check above
        args = argparse.Namespace(
            server=credentials.server,
            site_name=credentials.site,
            token_name=credentials.token_name,
            token=credentials.token,
            username=None,
            password=None,
            password_file=None,
            logging_level=None,
            no_certcheck=True,
            certificate=None,
            prompt=True,
            no_prompt=False,
            proxy=None,
            no_proxy=True,
            timeout=None,
            no_cookie=False,
        )
        test_session = Session()
        server = test_session.create_session(args)
        assert test_session.auth_token is not None
        assert test_session.site_id is not None
        assert test_session.user_id is not None
        E2EServerTests.saved_site_id = test_session.site_id
        return server

    def test_reuse_session(self):
        if not credentials:
            return
        # TODO current test command doesn't recognize skips - change to proper pytest
        # TODO and then we can get rid of the check above
        args = argparse.Namespace(
            server=None,
            site_name=None,
            token_name=None,
            token=None,
            username=None,
            password=None,
            password_file=None,
            logging_level=None,
            no_certcheck=True,
            certificate=None,
            prompt=True,
            no_prompt=False,
            proxy=None,
            no_proxy=True,
            timeout=None,
            no_cookie=False,
        )
        test_session = Session()
        test_session.create_session(args)
        assert test_session.auth_token is not None
        assert test_session.site_id is not None
        assert test_session.user_id is not None
        assert test_session.site_id == E2EServerTests.saved_site_id

    def test_read_password_file(self):
        if not credentials:
            return
        args = argparse.Namespace(
            server=credentials.server,
            site=credentials.site,
            token_name=credentials.token_name,
            token=None,
            username=None,
            password=None,
            password_file="users.csv",
            logging_level=None,
            no_certcheck=True,
            certificate=None,
            prompt=True,
            no_prompt=False,
            proxy=None,
            no_proxy=True,
            timeout=None,
            no_cookie=True,
        )
        test_session = Session()
        # Error: i/o operation on closed file ???
        # with self.assertRaises(SystemExit):
        # our file doesn't have a real token in it
        # test_session.create_session(args)

    def test_get_project(self):
        logger = log(__class__.__name__, "info")
        server = E2EServerTests.test_log_in()
        Server.get_project_by_name_and_parent_path(logger, server, "Default", None)


logging.disable(logging.NOTSET)



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log


class DeleteProjectCommand(Server):
    """
    Command to Delete the specified project from the server
    """

    name: str = "deleteproject"
    description: str = _("deleteproject.short_description")

    @staticmethod
    def define_args(delete_project_parser):
        delete_project_parser.add_argument("project_name", metavar="project-name", help=_("createproject.options.name"))
        set_parent_project_arg(delete_project_parser)

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        if args.parent_project_path:
            logger.debug("parent path: {}".format(args.parent_project_path))

        try:
            logger.debug(_("deleteproject.status").format(args.parent_project_path, args.project_name))
            project = Server.get_project_by_name_and_parent_path(
                logger, server, args.project_name, args.parent_project_path
            )
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, _("publish.errors.unexpected_server_response"), e)
        project_id = project.id

        try:
            logger.info(_("deleteproject.status").format(args.project_name))
            server.projects.delete(project_id)
            logger.info(_("common.output.succeeded"))
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, "tabcmd.result.failure.delete.project", e)



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.server import Server
from src.execution.global_options import *
from src.execution.localize import _
from src.execution.logger_config import log


class ListSiteCommand(Server):
    """
    Command to return a list of sites to which the logged in user belongs
    """

    name: str = "listsites"
    description: str = _("listsites.short_description")

    @staticmethod
    def define_args(list_site_parser):
        set_view_site_encryption(list_site_parser)

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        try:
            sites, pagination = server.sites.get()
            logger.info(_("listsites.status").format(session.username))
            for site in sites:
                print("NAME:", site.name)
                print("SITEID:", site.content_url)
                if args.get_extract_encryption_mode:
                    print("EXTRACTENCRYPTION:", site.extract_encryption_mode)
                print("")
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, e)



"""
Updated from https://github.com/mivek/prop2po to handle encoding
License: MIT
TO DO: contribute changes back and go back to installing from pypi
"""

import click


@click.command()
@click.argument('source', type=click.File('rt', encoding='utf-8'))
@click.argument('destination', type=click.File('wt', encoding='utf-8'))
@click.option('-l', '--language', type=click.STRING, help='The translation language')
@click.option('-p', '--project', type=click.STRING, help='The name of the project')
@click.option('-e', '--encoding', type=click.STRING, help='The encoding wanted')
def to_po(source, destination, encoding, language, project):
    """Converts a property file to a Gettext PO file.

    SOURCE is the path of the property file to convert.

    DESTINATION is the path of the Gettext PO file to create
    """

    header = """msgid ""
msgstr ""
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset={encoding}\\n"
"Content-Transfer-Encoding: 8bit\\n"
"X-Generator: prop2po\\n"
"Project-Id-Version: {project}\\n"
"Language: {language}\\n"
 # Copyright (C) YEAR Tableau Software 
 """
    lines = source.readlines()
    print(lines)
    destination.write(header.format(
        language=language,
        project=project,
        encoding=encoding
    ))
    for line in lines:
        if not line.isspace():
            parts = line.split('=')
            # TODO it fails on comments/lines with less than two parts after splitting
            destination.write('#:\n' + 'msgid "' + parts[0] + '"\n' 'msgstr "' + parts[1][:-1] + '"\n\n')


if __name__ == '__main__':
    to_po()


import sys

from src.execution.localize import _


class Constants:
    login_error = "401001"
    invalid_credentials = "401002"
    source_not_found = "404005"
    forbidden = "403022"
    resource_conflict_general = "409"
    source_already_exists = "409006"
    user_already_member_of_site = "409017"


class Errors:
    @staticmethod
    def is_expired_session(error):
        if hasattr(error, "code"):
            return error.code == Constants.invalid_credentials

    @staticmethod
    def is_resource_conflict(error):
        if hasattr(error, "code"):
            return error.code.startswith(Constants.resource_conflict_general)

    @staticmethod
    def is_login_error(error):
        if hasattr(error, "code"):
            return error.code == Constants.login_error

    @staticmethod
    def exit_with_error(logger, message=None, exception=None):
        try:
            if message and not exception:
                logger.error(message)
            if exception:
                if Errors.is_expired_session(exception):
                    logger.error(_("session.errors.session_expired"))
                    # TODO: add session as an argument to this method
                    #  and add the full command line as a field in Session?
                    # "session.session_expired_login"))
                    # session.renew_session()
                if message:
                    logger.debug(message)
                Errors.check_common_error_codes_and_explain(logger, exception)
        except Exception as exc:
            print("Error during log call from exception - {} {}".format(exc.__class__, message))
        sys.exit(1)

    @staticmethod
    def check_common_error_codes_and_explain(logger, error):
        logger.error(error)



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.server import Server
from src.execution.localize import _
from src.execution.logger_config import log


class DeleteSiteCommand(Server):
    """
    Command to delete a site
    """

    name: str = "deletesite"
    description: str = _("deletesite.short_description")

    @staticmethod
    def define_args(delete_site_parser):
        delete_site_parser.add_argument("site_name", help="name of site to delete")

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        site_id = server.sites.get_by_name(args.site_name)
        if site_id == session.site_id:
            Errors.exit_with_error(logger, "Cannot delete the site you are logged in to")
        try:
            server.sites.delete(site_id)
            logger.info("Successfully deleted the site")
        except TSC.ServerResponseError as e:
            Errors.exit_with_error(logger, "Error deleting site", e)



from src.commands.server import Server
from src.execution.localize import _
from src.execution.logger_config import log
from .session import Session


class LoginCommand(Server):
    """
    Logs in a Tableau Server user.
    """

    name: str = "login"
    description: str = _("login.short_description")

    @staticmethod
    def define_args(parser):
        # just uses global options
        pass

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        session.create_session(args)



import tableauserverclient as TSC

from src.commands.auth.session import Session
from src.commands.constants import Errors
from src.commands.server import Server
from src.execution.localize import _
from src.execution.logger_config import log


class CreateGroupCommand(Server):
    """
    This command is used to create a group
    """

    name: str = "creategroup"
    description: str = _("creategroup.short_description")

    @staticmethod
    def define_args(create_group_parser):
        create_group_parser.add_argument("name")

    @staticmethod
    def run_command(args):
        logger = log(__class__.__name__, args.logging_level)
        logger.debug(_("tabcmd.launching"))
        session = Session()
        server = session.create_session(args)
        try:
            logger.info(_("creategroup.status").format(args.name))
            new_group = TSC.GroupItem(args.name)
            server.groups.create(new_group)
            logger.info(_("tabcmd.result.succeeded"))
        except TSC.ServerResponseError as e:
            if args.continue_if_exists and Errors.is_resource_conflict(e):
                logger.info(_("tabcmd.result.already_exists.group").format(args.name))
                return
            Errors.exit_with_error(logger, "tabcmd.result.failed.create_group")



import unittest
from unittest import mock

from src.commands.user.remove_users_command import RemoveUserCommand
from .common_setup import *

commandname = "removeusers"


class RemoveUsersParserTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.parser_under_test = initialize_test_pieces(commandname, RemoveUserCommand)

    def test_remove_users_parser_required_name(self):
        with mock.patch("builtins.open", mock.mock_open(read_data="test")) as open_file:
            mock_args = [commandname, "group-name", "--users", "file"]
            args = self.parser_under_test.parse_args(mock_args)
            self.assertEqual(args.name, "group-name")

    def test_remove_users_parser_users_file(self):
        with mock.patch("builtins.open", mock.mock_open(read_data="test")) as open_file:
            mock_args = [commandname, "group-name", "--users", "users.csv"]
            args = self.parser_under_test.parse_args(mock_args)
            self.assertEqual(args.name, "group-name")
            open_file.assert_called_with("users.csv", "r", -1, "UTF-8", None)

    def test_remove_users_parser_missing_group_name(self):
        with mock.patch("builtins.open", mock.mock_open(read_data="test")):
            mock_args = [commandname, "--users", "users.csv"]
            with self.assertRaises(SystemExit):
                self.parser_under_test.parse_args(mock_args)

    # --[no-]complete?
